//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue;

import figue.event.PrintListenerInterface;
import figue.event.PrintEvent;
import figue.event.EventMulticaster;
import figue.path.PathInterface;
import figue.path.Path;
import figue.path.InsertGlyph;
import figue.resource.WrongResourceException;
import figue.resource.FontInterface;
import figue.resource.ColorInterface;
import figue.resource.PortableFontInterface;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.StopFormattingSonsException;
import figue.format.TooSmallPageException;
import figue.format.ActionNone;
import figue.format.ActionNewLine;
import figue.format.ActionInterface;
import figue.error.ImplementationError;
import figue.error.ShouldNotBeCalledError;
import figue.multithread.StoppedFacadeException;
import figue.box.GlyphInterface;
import figue.box.Atom;
import figue.box.Paragraph;
import figue.box.LightweightInterface;
import figue.internal.Cursor;
import figue.internal.AvailableVersion;
import figue.internal.AwtPsContext;
import figue.internal.ContextInterface;
import figue.internal.Globals;
import figue.internal.GlyphViewerInterface;

import java.awt.PrintJob;
import java.awt.Graphics;
import java.util.Vector;
import java.io.InputStream;
import antlr.ParserException;
import java.io.IOException;

/**
 * [PENDING: update the comments] PSFacade.
 */

public final class PSFacade extends Facade {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/PSFacade.java,v $ $revision$ $Date: 1998/11/02 15:54:33 $ Copyright 1998 INRIA.";

    public PSFacade(PrintJob aPrintJob,
		    FacadeInitializer anInitializer) {
	super(AvailableVersion.PRINTING.clone(null),anInitializer);
	internalInit(aPrintJob,anInitializer);
    }

    public PSFacade(PrintJob aPrintJob) {
	this(aPrintJob,new FacadeInitializer());
    }

    public PSFacade(PrintJob aPrintJob,
		    InputStream anInput) throws ParserException, IOException {
	this(aPrintJob,new FacadeInitializer(),anInput);
    }

    public PSFacade(PrintJob aPrintJob,
		    FacadeInitializer anInitializer,
		    InputStream anInput) throws ParserException, IOException {
	super(AvailableVersion.PRINTING.clone(null),anInitializer,anInput);
	internalInit(aPrintJob,anInitializer);
    }

    public PSFacade(PrintJob aPrintJob,
		    FacadeInitializer anInitializer,
		    FacadeInterface aFacade) throws WrongResourceException, CloneNotSupportedException {
	super(AvailableVersion.PRINTING.clone(null),anInitializer,aFacade);
	internalInit(aPrintJob,anInitializer);
    }

    private final void internalInit(PrintJob aPrintJob,
				    FacadeInitializer anInitializer) {
	_context = new PSContext(this,anInitializer.getInitialFont(),anInitializer.getInitialColor(),anInitializer.getInitialBackground(),aPrintJob);
	getVersion().setContext(_context);
	_rectangleCacheForBoundingBox = new Rectangle();
	_rectangleCacheForRefreshZone = new Rectangle();
	_alternative = new PaginationAlternative();
	_leftMargin = 0;
	_rightMargin = 0;
	_topMargin = 0;
	_bottomMargin = 0;
	_printListeners = null;
	init();
    }

    public final void setMargins(int aLeftMargin, int aRightMargin, int aTopMargin, int aBottomMargin) {
	_leftMargin = aLeftMargin;
	_rightMargin = aRightMargin;
	_topMargin = aTopMargin;
	_bottomMargin = aBottomMargin;
    }
  
    final GlyphViewerInterface getViewer() {
	reportError(new ShouldNotBeCalledError());
	return null;
    }
  
    public final ContextInterface getContext() {
	return _context;
    }

    public final void refresh(Rectangle aRectangle) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void refresh() {
	reportError(new ShouldNotBeCalledError());
    }

    public final void addPrintListener(PrintListenerInterface aListener) {
	_printListeners = EventMulticaster.addPrintListener(_printListeners,aListener);
    }
  
    public final void removePrintListener(PrintListenerInterface aListener) {
	_printListeners = EventMulticaster.removePrintListener(_printListeners,aListener);
    }

    protected final void insureVisibleWhenInactive(PathInterface aPath) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void print(PrintJob aPrintJob) {
	_context.setPrintJob(aPrintJob);
	print();
    }

    public final void print() {
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("-> print");
	}
	try {
// 	    _context.awaitClean(getRootGlyph());
	    while ( ! getRootGlyph().isClean() ) {
		getInactive().await();
	    }
	    synchronized ( getDrawable() ) {
		if ( _printListeners != null ) {
		    _printListeners.printInitiate(new PrintEvent(this,0,0));
		}
		_context.awaitClean(getRootGlyph());
		final Dimension theWholeSize = getRootGlyph().getSizeValue();
		final int theWholeHeight = theWholeSize.getHeight();
		final int theWholeWidth = theWholeSize.getWidth();
		final int x = 0;
		int y = 0;
		int theCurrentPageNumber = 1;
		int theHeightDone = 0;
		int thePercentDone = 0;
		int theNextCut = 0;
		final int thePageWidth = _context.getRefreshZone().getWidth();
		computePagination();
	    main_loop: while ( true ) {
		synchronized ( this ) {
		    if ( getThreadState().isDrawable() ) {
			getThreadState().startDraw();
			while ( true ) {
			    theNextCut = getNextCut();
			    _context.setRefreshZone(_leftMargin,_topMargin,_rightMargin,_bottomMargin);
			    _context.setClipBounds(x,y,thePageWidth,theNextCut);
			    executeDrawTask();
			    theCurrentPageNumber++;
			    theHeightDone+=theNextCut;
			    thePercentDone = ( 100 * theHeightDone ) / theWholeHeight;
			    y = theHeightDone;
			    if ( theHeightDone >= theWholeHeight ) {
				break main_loop;
			    }
			    else {
				if ( _printListeners != null ) {
				    _printListeners.printAdvance(new PrintEvent(this,theCurrentPageNumber - 1,thePercentDone));
				}
			    }
			}
		    }
		}
		getDrawable().await();
	    }
		if ( _printListeners != null ) {
		    _printListeners.printDone(new PrintEvent(this,theCurrentPageNumber - 1,100));
		}
	    }
	    endDraw();
	}
	catch ( StoppedFacadeException anException ) {
	    return;
	}
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("<- print");
	}    
    }

    protected final void executeDrawTask() {
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("\nbegin draw\n");
	}
	GlyphInterface theRoot = getRootGlyph();
	_rectangleCacheForRefreshZone.setBounds(_context.getClipBounds());
	_rectangleCacheForBoundingBox.setBounds(0,0,0,0);
	getVersion().draw(theRoot,null,getSelectionPath(),
			  _rectangleCacheForRefreshZone,_rectangleCacheForBoundingBox);
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("\nend draw\n");
	}
    }

    private final void computePagination() {
	final Dimension theWholeSize = getRootGlyph().getSizeValue();
	final int theWholeHeight = theWholeSize.getHeight();
	_alternative.findAllCuts(getRootGlyph());
	final Rectangle thePageSize = _context.getRefreshZone();
	final int thePageHeight = thePageSize.getHeight();
	final FacadeInitializer theInitializer = new FacadeInitializer();
	theInitializer.setFormattingWidth(thePageHeight);
	ASCIIFacade theFacade = new ASCIIFacade(theInitializer);
	PathInterface thePath = new Path();
	ParagraphPrint theDocument = new ParagraphPrint(new Measure(0,UnitInterface.PIXEL),
							new Measure(0,UnitInterface.PIXEL),
							new Measure(0,UnitInterface.PIXEL)) ;
	thePath.addOperation(new InsertGlyph(theDocument,0));
	ActionNewLine[] theCuts = _alternative.getAllCuts();
	final int theSize = theCuts.length;
	StringBuffer theToken = new StringBuffer();
	boolean theMakingTokenFlag = false;
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    switch ( theCuts[theRank].getCost() ) {
	    case Globals.BEST: {
		if ( theMakingTokenFlag == true ) {
		    theDocument.addChild(Atom.newAtom(theToken.toString()));

		    theMakingTokenFlag = false;
		    theToken.setLength(0);
		}
		theDocument.addChild(Atom.newAtom('!'));
		break;
	    }
	    case Globals.GOOD: {
		theMakingTokenFlag = true;
		theToken.append('+');
		break;
	    }
	    case Globals.BAD: {
		theMakingTokenFlag = true;
		theToken.append('?');
		break;
	    }
	    case Globals.FORBIDDEN: {
		theMakingTokenFlag = true;
		theToken.append('-');
		break;
	    }
	    default: {
		reportError(new ImplementationError("Unexpected"));
		break;
	    }
	    }
	}
	theFacade.buildInit(thePath);
	theFacade.buildComplete();
	theFacade.update(null);
	_pagination = theDocument;
	_currentPageRank = 0;
	theFacade.disposeWhenInactive();
    }

    private final int getNextCut() {
	return _pagination.getCutsForPrint().getChild(_currentPageRank++).getSizeValue().getWidth();
    }

    protected final void init() {
	_context.init();
    }

    protected final void dispose() {
	_context.dispose();
	_context = null;
	_rectangleCacheForBoundingBox = null;
	_rectangleCacheForRefreshZone = null;
	_alternative.dispose();
	_alternative = null;
	_pagination = null;
	_printListeners = null;
	super.dispose();
    }
    
    private PSContext _context;
    private Rectangle _rectangleCacheForBoundingBox;
    private Rectangle _rectangleCacheForRefreshZone;
    private PaginationAlternative _alternative;
    private ParagraphPrint _pagination;
    private int _currentPageRank;
    private int _leftMargin;
    private int _rightMargin;
    private int _topMargin;
    private int _bottomMargin;
    private PrintListenerInterface _printListeners;
  
}

class PaginationAlternative implements AlternativeInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/PSFacade.java,v $ $revision$ $Date: 1998/11/02 15:54:33 $ Copyright 1998 INRIA.";
  
    public PaginationAlternative() {
	_glyph = null;
	_decalage = null;
	_action = ActionNone.INSTANCE;
	_cuts = null;
	_rectangleCache = new Rectangle();
	_transformerCache = new Transformer();
    }

    public final void findAllCuts(GlyphInterface aGlyph) {
	final int theHeight = aGlyph.getSizeValue().getHeight();
	_cuts = new ActionNewLine[theHeight];
	for ( int i = 0; i < _cuts.length; i++ ) {
	    _cuts[i] = new ActionNewLine(i,Globals.NOT_SEEN);
	}
	aGlyph.computeCuttingCosts(this,_rectangleCache,_transformerCache);
	for ( int i = 0; i < _cuts.length; i++ ) {
	    if ( _cuts[i].getCost() == Globals.NOT_SEEN ) {
		_cuts[i].setCost(Globals.BAD);
	    }
	}
    }

    public final ActionNewLine[] getAllCuts() {
	return _cuts;
    }
  
    public final void execute() 
	throws TooSmallPageException, StopFormattingSonsException {
	if ( Globals.DEBUG_FORMAT_HISTORY ) {
	    System.out.println("-> execute : " + toString()); 
	}
	_action.execute(_glyph,_decalage,this);
    }
  
    public final void tryBetter(ActionInterface anAction,
				GlyphInterface aGlyph,
				DecalageInterface aDecalage) {
	if ( anAction.isComplient(aGlyph) && 
	     anAction.isBetterThan(_action) ) {
	    _glyph = aGlyph;
	    _decalage = aDecalage;
	    _action = anAction;
	}
    }

    public final void setAction(ActionInterface anAction) {
	_action = anAction;
    }

    public final void preComputeActionCostNewLine(ActionNewLine anAction) {
	final int theIndex = anAction.getIndex();
	final ActionNewLine theOldAction = _cuts[theIndex];
	if ( ! theOldAction.isBetterThan(anAction) ) {
	    _cuts[theIndex] = anAction;
	}
    }

    public final void dispose() {
	_glyph = null;
	_decalage = null;
	_action = null;
	_cuts = null;
	_rectangleCache = null;
	_transformerCache = null;
    }
  
    public final String toString() {
	StringBuffer theResult = new StringBuffer("PaginationAlternative<");
	if ( _action != null ) {
	    theResult.append(_action);
	}
	if ( _glyph != null ) {
	    theResult.append(_glyph.toString());
	}
	theResult.append(">");
	return theResult.toString();
    }
  
    private GlyphInterface _glyph;
    private DecalageInterface _decalage;
    private ActionInterface _action;
    private ActionNewLine[] _cuts;
    private Rectangle _rectangleCache;
    private Transformer _transformerCache;
  
}

final class ParagraphPrint extends Paragraph {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/PSFacade.java,v $ $revision$ $Date: 1998/11/02 15:54:33 $ Copyright 1998 INRIA.";
  
    public ParagraphPrint(MeasureInterface anInterWord,
			  MeasureInterface anInterline,
			  MeasureInterface anIdentation) {
	super(anInterWord,anInterline,anIdentation);
    }

    public final GlyphInterface getCutsForPrint() {
	return getComponent();
    }
  
}

final class PSContext extends AwtPsContext {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/PSFacade.java,v $ $revision$ $Date: 1998/11/02 15:54:33 $ Copyright 1998 INRIA.";

    public  PSContext(PSFacade aFacade,
		      String aFontName,
		      String aColorName,
		      String aBackgroundName,
		      PrintJob aPrintJob) {
	super(aFacade,aFontName,aColorName,aBackgroundName);
	_printJob = aPrintJob;
	_printGraphics = null;
	_singleChar = new char[1];
	_clipBounds = new Rectangle();
	_refreshZone = new Rectangle();
    }

    public final void init() {
	if ( _printGraphics == null ) {
	    if ( _printJob == null ) {
		System.out.println("*** PLANTE null printjob ***");
	     
	    }
	    else {
		_printGraphics = _printJob.getGraphics();
		_printGraphics.setFont(getInitialFont().getAwtFont());
	    }
	}
	if ( _printGraphics != null ) {
	    getFacade().initDone();
	    getFacade().setFormattingWidth(Math.min(getRefreshZone().getWidth(),
						    getFacade().getFormattingWidth())); /// WHY HERE ??
	}
	else {
	    getFacade().fatalError();
	}
    }
    
    public final void setCursor(Cursor aCursor) {
    }

    public final void setAwtContext(Graphics aGraphics) {
        reportError(new ShouldNotBeCalledError());
    }

    public final void setFont(FontInterface aFont) {
	super.setFont(aFont);
	/// SHOULD TEST ONLY FOR DEFERED FONT INITIALIZATION ...
	if ( _printGraphics != null ) {
	    _printGraphics.setFont(aFont.getAwtFont());
	}
    }
  
    public final void setColor(ColorInterface aColor) {
	super.setColor(aColor);
	_printGraphics.setColor(aColor.getAwtColor()); 
    }

    protected final java.awt.FontMetrics getAwtFontMetrics(java.awt.Font aFont) {
	return _printGraphics.getFontMetrics(aFont);
    }

    public final void drawBytes(byte[] aByteArray, int x, int y) {
	_printGraphics.drawBytes(aByteArray,0,aByteArray.length,x,y);
    }

    public final void drawString(String aString, int x, int y) {
	_printGraphics.drawString(aString,x,y);
    }

    public final void drawSymbol(PortableFontInterface aFont, String aString, int x, int y) {
	aFont.draw(_printGraphics,aString,x,y);
    }

    public final void drawChars(char[] aCharArray, int x, int y) {
	_printGraphics.drawChars(aCharArray,0,aCharArray.length,x,y);
    }

    public final void drawChar(char aChar, int x, int y) {
	_singleChar[0] = aChar;
	_printGraphics.drawChars(_singleChar,0,1,x,y);
    }

    public final void fillRectangle(int x, int y, int width, int height) {
	_printGraphics.setColor(getBackground().getAwtColor()); 
	_printGraphics.fillRect(x,y,width,height);
	_printGraphics.setColor(getColor().getAwtColor());
    }

    public final void drawRectangle(int x, int y, int width, int height, ColorInterface aColor) {
	_printGraphics.setColor(aColor.getAwtColor());
	_printGraphics.drawRect(x,y,width,height);
	_printGraphics.setColor(getColor().getAwtColor());
    }

    public final void drawLine(int x1, int y1, int x2, int y2) {
	_printGraphics.drawLine(x1,y1,x2,y2);
    }
  
    public final void drawLightweightComponent(LightweightInterface aLightweight, int x, int y) {
	aLightweight.setForeground(getColor().getAwtColor());
	aLightweight.setBackground(getBackground().getAwtColor());
	aLightweight.setFont(getFont().getAwtFont());
	aLightweight.setLocation(0,0);
	_printGraphics.translate(x,y);
	aLightweight.update(_printGraphics);
	_printGraphics.translate(-x,-y);
    }

    public final void applyFontToLightweightComponent(FontInterface aFont,
						      LightweightInterface aLightweight) {
	aLightweight.setFont(aFont.getAwtFont());
    }

    public final void setPrintJob(PrintJob aPrintJob) {
	_printJob = aPrintJob;
	if ( aPrintJob != null ) {
	    flush();
	}
    }

    public final Rectangle getRefreshZone() {
	final java.awt.Dimension thePage = _printJob.getPageDimension();
	_refreshZone.setBounds(0,0,thePage.width,thePage.height);
	return _refreshZone;
    }

    public final Rectangle getClipBounds() {
	return _clipBounds;
    }
  
    public final void setClipBounds(int x, int y, int width,int height) {
	_clipBounds.setBounds(x,y,width,height);
	_printGraphics.setClip(0,0,width,height);
	_printGraphics.translate(-x,-y);
    }

    public final void setRefreshZone(int x, int y, int width,int height) {
	
    }

    public final void flush() {
	if ( _printGraphics != null ) {
	    _printGraphics.dispose();
	}
	_printGraphics = _printJob.getGraphics();
	_printGraphics.setColor(getBackground().getAwtColor());
	_printGraphics.fillRect(getClipBounds().getX(),getClipBounds().getY(),getClipBounds().getWidth(),getClipBounds().getHeight());
	_printGraphics.setFont(getFont().getAwtFont());
	_printGraphics.setColor(getColor().getAwtColor());
    }

    public final void touch() {
    }

    public final void dispose() {
	if ( _printJob != null ) {
	    _printJob.end();
	}
	_printJob = null;
	if ( _printGraphics != null ) {
	    _printGraphics.dispose();
	}
	_printGraphics = null;
	_singleChar = null;
	_clipBounds = null;
	_refreshZone = null;
	super.dispose();
    }

    private PrintJob _printJob;
    private Graphics _printGraphics;
    private char[] _singleChar;
    private Rectangle _clipBounds;
    private Rectangle _refreshZone;
  
}
