//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue;

import figue.resource.FontMetricInterface;
import figue.box.FlyWeightChar;
import figue.box.FlyWeightAtomInterface;
import figue.internal.ContextInterface;

/**
 * [PENDING: update the comments] UnitInterface.
 */

public interface UnitInterface extends Cloneable { 

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/UnitInterface.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    public boolean setCurrentFontMetric(ContextInterface aContext,
					FontMetricInterface aFontMetric);
  
    public int convertValueInPixel(int aValue);
    public String getName();

    public void dispose();
    public Object clone() throws CloneNotSupportedException;

    public final static UnitInterface PIXEL = new UnitPixel();
    public final static UnitInterface BLANK_TENTH = new UnitBlankTenth();
    public final static UnitInterface LINE_TENTH = new UnitLineTenth();

}

final class UnitPixel implements UnitInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/UnitInterface.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";
  
    UnitPixel() {
    }

    public final boolean setCurrentFontMetric(ContextInterface aContext,
					      FontMetricInterface aFontMetric) {
	return false;
    }
  
    public final int convertValueInPixel(int aValue) {
	return aValue;
    }

    public final void dispose() {
	;
    }

    public final Object clone() {
	return this;
    }
  
    public final String toString() {
	return getName();
    }
    public final String getName() {
	return "Pixel";
    }

}

abstract class UnitTenth implements UnitInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/UnitInterface.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    protected UnitTenth(char aChar) {
	_flyWeight = new FlyWeightChar(aChar);
    }

    protected final void finalize() {
	if ( _flyWeight != null ) {
	    _flyWeight.dispose();
	}
    }

    public final void dispose() {
	_flyWeight.dispose();
	_flyWeight= null;
    }

    public final boolean setCurrentFontMetric(ContextInterface aContext,
					      FontMetricInterface aFontMetric) {
	FlyWeightAtomInterface theOldFlyWeight = _flyWeight;
	_flyWeight = _flyWeight.format2D(aContext,aFontMetric,null);
	return ( theOldFlyWeight != _flyWeight );
    }

    protected final FlyWeightAtomInterface getFlyWeight() {
	return _flyWeight;
    }

    public Object clone() {
	return this;
    }

    private FlyWeightAtomInterface _flyWeight;
  
}

final class UnitBlankTenth  extends UnitTenth {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/UnitInterface.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";
  
    public UnitBlankTenth() {
	super('O');
    }

    public final int convertValueInPixel(int aValue) {
	return ( ( aValue * getFlyWeight().getSizeValue().getWidth() ) / 10 );
    }

    public final Object clone() {
	return new UnitBlankTenth();
    }

    public final String toString() {
	return getName();
    }
    public final String getName() {
	return "Blank10e";
    }

}

final class UnitLineTenth extends UnitTenth {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/UnitInterface.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";
  
    public UnitLineTenth() {
	super('O');
    }

    public final int convertValueInPixel(int aValue) {
	return ( ( aValue * getFlyWeight().getSizeValue().getHeight() ) / 10 );
    }

    public final Object clone() {
	return new UnitLineTenth();
    }

    public final String toString() {
	return getName();
    }
    public final String getName() {
	return "Line10e";
    }

}
