//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.resource.FontOwnerInterface;
import figue.resource.ColorOwnerInterface;
import figue.resource.BackgroundOwnerInterface;
import figue.resource.ColorDefered;
import figue.resource.FontDefered;
import figue.resource.ColorInterface;
import figue.resource.FontInterface;
import figue.resource.FontInitializer;
import figue.resource.ColorInitializer;
import figue.resource.BackgroundInitializer;

import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

/**
 * [PENDING: update the comments] ChangeGraphicalContextAnonymous.
 */

public final class ChangeGraphicalContextAnonymous extends ChangeGraphicalContextAbstract implements FontOwnerInterface, ColorOwnerInterface, BackgroundOwnerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/ChangeGraphicalContextAnonymous.java,v $ $revision$ $Date: 1998/11/02 15:54:35 $ Copyright 1998 INRIA.";

    public ChangeGraphicalContextAnonymous(String aFontName,
					   String aColorName,
					   String aBackgroundName) {
	super();
	if ( aFontName != null ) {
	    _font = new FontDefered(aFontName,new FontInitializer(this));
	}
	else {
	    _font = null;
	}
	if ( aColorName != null ) {
	    _color = new ColorDefered(aColorName,new ColorInitializer(this));
	}
	else {
	    _color = null;
	}
	if ( aBackgroundName != null ) {
	    _background = new ColorDefered(aBackgroundName,new BackgroundInitializer(this));
	}
	else {
	    _background = null;
	}
    }
  
    public ChangeGraphicalContextAnonymous(String aFontName,
					   String aColorName) {
	this(aFontName,aColorName,null);
    }
  
    public ChangeGraphicalContextAnonymous(String aFontName) {
	this(aFontName,null,null);
    }
  
    public ChangeGraphicalContextAnonymous(FontInterface aFont, 
					   ColorInterface aColor,
					   ColorInterface aBackground) {
	super();
	_font = aFont;
	_color = aColor;
	_background = aBackground;
    }

    public ChangeGraphicalContextAnonymous(FontInterface aFont) {
	this(aFont,null,null);
    }

    public ChangeGraphicalContextAnonymous(ColorInterface aColor) {
	this(null,aColor,null);
    }

    public ChangeGraphicalContextAnonymous(FontInterface aFont, 
					   ColorInterface aColor) {
	this(aFont,aColor,null);
    }

    public ChangeGraphicalContextAnonymous(ColorInterface aColor,
					   ColorInterface aBackground) {
	this(null,aColor,aBackground);
    }

    public Object clone() throws CloneNotSupportedException {
	final ChangeGraphicalContextAnonymous theResult = (ChangeGraphicalContextAnonymous) super.clone();
	if (  _font != null ) {
	    theResult._font = new FontDefered(_font.getName(),new FontInitializer(theResult));
	}
	else {
	    theResult._font = null;
	}
	if ( _color != null ) {
	    theResult._color = new ColorDefered(_color.getName(),new ColorInitializer(theResult));
	}
	else {
	    theResult._color = null;
	}
	if ( _background != null ) {
	    theResult._background = new ColorDefered(_background.getName(),new BackgroundInitializer(theResult));
	}
	else {
	    theResult._background = null;
	}
	return theResult;
    }
    
    public final FontInterface getFont() {
	return _font;
    }

    public final void setFont(FontInterface aFont) {
	_font = aFont;
    }
  
    public final ColorInterface getColor() {
	return _color;
    }

    public final void setColor(ColorInterface aColor) {
	_color = aColor;
    }

    public final ColorInterface getBackground() {
	return _background;
    }

    public final void setBackground(ColorInterface aColor) {
	_background = aColor;
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("ChangeGraphicalContext<");
	if ( getFont() != null ) {
	    theResult.append(getFont().toString());
	}
	if ( getColor() != null ) {
	    theResult.append(getColor().toString());
	}
	if ( getBackground() != null ) {
	    theResult.append(getBackground().toString());
	}
	theResult.append(super.toString()).append(">");
	return theResult.toString();
    }

    
    protected final void writeIntrinsicState(Writer anOutput) throws IOException {
	super.writeIntrinsicState(anOutput);
	if ( _font != null ) {
	    anOutput.write(" Font=\"");
	    Utility.writeStringAsXmlValue(anOutput,_font.getName());
	    anOutput.write("\"");
	}
	if ( _color != null ) {
	    anOutput.write(" Foreground=\"");
	    Utility.writeStringAsXmlValue(anOutput,_color.getName());
	    anOutput.write("\"");
	}
	if ( _background != null ) {
	    anOutput.write(" Background=\"");
	    Utility.writeStringAsXmlValue(anOutput,_background.getName());
	    anOutput.write("\"");
	}
    }

    private FontInterface _font;
    private ColorInterface _color;
    private ColorInterface _background;

}

