//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.resource.FontMetricInterface;
import figue.selection.HighlighterInterface;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategySetSize;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategyComputeSizeTill;
import figue.format.StopFormattingSonsException;
import figue.format.ComputedValueInterface;
import figue.format.ActionNewLine;
import figue.format.ModificationInterface;
import figue.format.SonModificationStatus;
import figue.error.ErrorHandlerInterface;
import figue.io.ExternalizableAsXMLInterface;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;
import figue.internal.BooleanWrapper;
import figue.internal.Direction;
import figue.internal.PairGlyphPath;
import figue.FacadeInterface;
import figue.InteractiveFacade;
import figue.internal.Cursor;

import java.io.Writer;
import java.io.IOException;
import java.awt.event.MouseEvent;

/**
 * [PENDING: update the comments] GlyphInterface.
 */

public abstract class GlyphInterface implements ComputedValueInterface, ErrorHandlerInterface, ExternalizableAsXMLInterface, Cloneable {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/GlyphInterface.java,v $ $revision$ $Date: 1998/11/02 15:54:39 $ Copyright 1998 INRIA.";
  
    protected GlyphInterface() {
	super();
    }

    public Object clone() throws CloneNotSupportedException {
	final GlyphInterface theResult = (GlyphInterface) super.clone();
	return theResult;
    }

    public abstract GlyphInterface getParent();
    public abstract GlyphInterface getChild(int aRank);
    public abstract void addChild(GlyphInterface aGlyph);
    public abstract void insertChild(int aRank, GlyphInterface aGlyph);
    public abstract void replaceChild(int aRank, GlyphInterface aGlyph);
    public abstract void removeChild();
    public abstract void removeChild(int aRank);
    public abstract int indexOf(GlyphInterface aGlyph);
    public abstract int numberOfChildren();

    protected abstract void setParent(GlyphInterface aGlyph);
    protected abstract void clearParent();
  
    protected abstract void refresh();
    /**
     * [PENDING: update the comments] draw2D
     * @level advanced
     */
    public abstract boolean draw2D(ContextInterface aContext,
				   Transformer aTransformer,
				   SelectionPathInterface aSelectionPath,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox);
    protected abstract void draw2DClean(ContextInterface aContext,
					Transformer aTransformer,
					SelectionPathInterface aSelectionPath,
					Rectangle aRefreshZone,
					Rectangle aBoundingBox);
    protected abstract boolean draw2D(ContextInterface aContext,
				      Transformer aTransformer,
				      Rectangle aRefreshZone,
				      Rectangle aBoundingBox);
    protected abstract void draw2DClean(ContextInterface aContext,
					Transformer aTransformer,
					Rectangle aRefreshZone,
					Rectangle aBoundingBox);
    protected abstract ContextMementoInterface draw2DSelections(ContextInterface aContext,
								Transformer aTransformer,
								SelectionPathInterface aSelectionPath);
    public abstract HighlighterInterface getHighlighter(SelectionPathInterface aSelectionPath);
    protected abstract SelectionPathInterface zoomInSelection(SelectionPathInterface aSelectionPath,
							      int aRank);
    /**
     * [PENDING: update the comments] getDeepestContains
     * @level advanced
     */
    public abstract PairGlyphPath getDeepestContains(int x,
						     int y, 
						     Rectangle aBoundingBox,
						     ContextInterface aContext);
    protected abstract PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath);
    protected abstract boolean isFinalTarget();
    /**
     * [PENDING: update the comments] redispatchEvent
     * @level advanced
     */
    public abstract boolean redispatchEvent(InteractiveFacade aFacade,
					    int aMouseEventType,MouseEvent anEvent);
    /**
     * [PENDING: update the comments] 
     * @level advanced
     */
    public abstract Cursor getCursor();
    protected abstract void setCursor(Cursor aCursor);

    protected abstract Dimension computeSize(ContextInterface aContext,
					     StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
					   
					     StrategySetOrigin aSetOriginStrategy,
					     StrategyComputeSizeTill aComputeSizeTillStrategy);
  
    /**
     * [PENDING: update the comments] format2D
     * @level advanced
     */
    public abstract void format2D(ContextInterface aContext,
				  FontMetricInterface aFontMetric,
				  AlternativeInterface anAlternative,
				  DistanceInterface aDistance,
				  StrategyReadyToFormat aReadyToFormatStrategy,
				  StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
				  StrategySetOrigin aSetOriginStrategy,
				  StrategySetSize aSetSizeStrategy,
				  boolean aMustFormatFlag)
	throws StopFormattingSonsException;
    /**
     * [PENDING: update the comments] computeDistance2D
     * @level advanced
     */
    public abstract int computeDistance2D(DistanceInterface aDistance,
					  DecalageInterface aDecalage);
    /**
     * [PENDING: update the comments] computeDecalage2D
     * @level advanced
     */
    public abstract int computeDecalage2D(GlyphInterface aGlyph,
					  DecalageInterface aDecalage);
    /**
     * [PENDING: update the comments] computeInitialDecalage2D
     * @level advanced
     */
    public abstract int computeInitialDecalage2D(DecalageInterface aDecalage);
    /**
     * [PENDING: update the comments] tryAlternativeForInitial
     * @level advanced
     */
    public abstract void tryAlternativeForInitial(AlternativeInterface anAlternative);
    /**
     * [PENDING: update the comments] tryAlternativeForDistance
     * @level advanced
     */
    public abstract void tryAlternativeForDistance(DistanceInterface aDistance,
						   AlternativeInterface anAlternative);
    /**
     * [PENDING: update the comments] tryAlternativeForDecalage
     * @level advanced
     */
    public abstract void tryAlternativeForDecalage(DecalageInterface aDecalage,
						   AlternativeInterface anAlternative);
    /**
     * [PENDING: update the comments] isActionNoneComplient
     * @level advanced
     */
    public abstract boolean isActionNoneComplient();
    /**
     * [PENDING: update the comments] isActionNewLineComplient
     * @level advanced
     */
    public abstract boolean isActionNewLineComplient(ActionNewLine anAction);
    /**
     * [PENDING: update the comments] isActionShrinkComplient
     * @level advanced
     */
    public abstract boolean isActionShrinkComplient();
    /**
     * [PENDING: update the comments] isActionShrinkSonComplient
     * @level advanced
     */
    public abstract boolean isActionShrinkSonComplient();
    /**
     * [PENDING: update the comments] executeActionNewLine
     * @level advanced
     */
    public abstract void executeActionNewLine(GlyphInterface aGlyph) throws StopFormattingSonsException;
    /**
     * [PENDING: update the comments] executeActionShrink
     * @level advanced
     */
    public abstract void executeActionShrink() throws StopFormattingSonsException;
    /**
     * [PENDING: update the comments] executeActionShrinkSon
     * @level advanced
     */
    public abstract void executeActionShrinkSon();

    public abstract int getOriginX();
    public abstract int getOriginY();
    /**
     * [PENDING: update the comments] setOrigin
     * @level advanced
     */
    public abstract void setOrigin(int x, int y);
    /**
     * [PENDING: update the comments] resetOrigin
     * @level advanced
     */
    public abstract void resetOrigin(int x, int y, ContextInterface aContext);
    /**
     * [PENDING: update the comments] isClean
     * @level advanced
     */
    public abstract boolean isClean();
    /**
     * [PENDING: update the comments] isDirty
     * @level advanced
     */
    public abstract boolean isDirty();
    /**
     * [PENDING: update the comments] isNew
     * @level advanced
     */
    public abstract boolean isNew();
    /**
     * [PENDING: update the comments] isProceed
     * @level advanced
     */
    public abstract boolean isProceed();
    /**
     * [PENDING: update the comments] isUnstable
     * @level advanced
     */
    public abstract boolean isUnstable();
    /**
     * [PENDING: update the comments] setClean
     * @level advanced
     */
    public abstract void setClean();
    /**
     * [PENDING: update the comments] setProceed
     * @level advanced
     */
    public abstract void setProceed();
    /**
     * [PENDING: update the comments] setDirty
     * @level advanced
     */
    public abstract void setDirty();
    /**
     * [PENDING: update the comments] setNew
     * @level advanced
     */
    public abstract void setNew();
    /**
     * [PENDING: update the comments] setUnstable
     * @level advanced
     */
    public abstract void setUnstable();
  

    protected abstract void computeBoundingBox(Rectangle aRectangle,
					       Transformer aTransformer);
    /**
     * [PENDING: update the comments] computeBoundingBox
     * @level advanced
     */
    public abstract void computeBoundingBox(Rectangle aRectangle);
    /**
     * [PENDING: update the comments] computeOldBoundingBox
     * @level advanced
     */
    public abstract void computeOldBoundingBox(Rectangle aRectangle,
					       int anOldX,
					       int anOldY,
					       Dimension anOldSize);
  
    protected abstract void computeEstimatedBoundingBox(Rectangle aRectangle,
							Transformer aTransformer);
    protected abstract void computeEstimatedBoundingBox(Rectangle aRectangle);
  

    protected abstract ModificationInterface getModification();
    protected abstract void setModification(ModificationInterface aModification);
    protected abstract boolean sonModified(GlyphInterface aGlyph,
					   SonModificationStatus aStatus,
					   ContextInterface aContext);
    /**
     * [PENDING: update the comments] markAllBranchDirty
     * @level advanced
     */
    public abstract void markAllBranchDirty(SonModificationStatus aStatus,
					    ContextInterface aContext);
    /**
     * [PENDING: update the comments] cleanModificationAndUpdateRefreshZone
     * @level advanced
     */
    public abstract void cleanModificationAndUpdateRefreshZone(Rectangle aVisibleZone,
							       Rectangle aRefreshZone,
							       BooleanWrapper aRefreshZoneNotInitializedFlag,
							       Rectangle aRectangle,
							       ContextInterface aContext);
    protected abstract void cleanModification(ContextInterface aContext);

    protected abstract Direction checkRefreshZone(Rectangle aRefreshZone,
						  Rectangle aBoundingBox);
    protected abstract Direction checkRefreshZoneAgainstEstimation(Rectangle aRefreshZone,
								   Rectangle aBoundingBox);
    protected abstract void doTranslate(Rectangle aRectangle);
    protected abstract void undoTranslate(Rectangle aRectangle);
    /**
     * [PENDING: update the comments] doTranslatesZones
     * @level advanced
     */
    public abstract void doTranslatesZones(Rectangle aRefreshZone,
					   Rectangle aVisibleZone,
					   int aRank);
    /**
     * [PENDING: update the comments] undoTranslateZones
     * @level advanced
     */
    public abstract void undoTranslateZones(Rectangle aRefreshZone,
					    Rectangle aVisibleZone,
					    int aRank);
    /**
     * [PENDING: update the comments] computeCuttingCosts
     * @level advanced
     */
    public abstract void computeCuttingCosts(AlternativeInterface anAlternative,
					     Rectangle aRectangle,
					     Transformer aTransformer);
    /**
     * [PENDING: update the comments] improveEstimatedHeightAndCheckAwake
     * @level advanced
     */
    public abstract boolean improveEstimatedHeightAndCheckAwake(int anHeight);
    /**
     * [PENDING: update the comments] improveSizeEstimationAndUpdateParents
     * @level advanced
     */
    public abstract void improveSizeEstimationAndUpdateParents(ContextInterface aContext,
							       int aRank);
    protected abstract StrategyImproveSizeEstimation chooseAlgorithm(StrategyImproveSizeEstimation aStrategy);
    protected abstract void setSizeEstimation(Dimension aCurrentSize);
    /**
     * [PENDING: update the comments] getSizeEstimation
     * @level advanced
     */
    public abstract Dimension getSizeEstimation();

    protected abstract Dimension getSize();
    protected abstract Dimension getEagerSize();
    public abstract Dimension getSizeValue();
    /**
     * [PENDING: update the comments] setSize
     * @level advanced
     */
    public abstract void setSize(Dimension aSize);
    /**
     * [PENDING: update the comments] resetSize
     * @level advanced
     */
    public abstract void resetSize(Dimension aSize,ContextInterface aContext);
    /**
     * [PENDING: update the comments] getFacade
     * @level advanced
     */
    public abstract FacadeInterface getFacade();

    protected abstract void writeIntrinsicState(Writer anOutput) throws IOException;
    protected abstract boolean writeExtrinsicState(Writer anOutput) throws IOException;
    protected abstract String getName();
  
}

