//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.resource.FontMetricInterface;
import figue.geometry.PositionToFather;
import figue.geometry.PositionToBrother;
import figue.geometry.Point;
import figue.geometry.Dimension;
import figue.geometry.Size;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.SonModificationStatus;
import figue.format.ModificationInterface;
import figue.format.ModificationStatus;
import figue.format.Modification;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategyComputeSizeTill;
import figue.format.ActionShrink;
import figue.format.ActionShrinkSon;
import figue.internal.ContextInterface;
import figue.MeasureInterface;
import figue.UnitInterface;
import figue.Measure;

import java.util.Vector;
import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

/**
 * [PENDING: update the comments] HorizontalAbstract.
 * @level advanced
 */

public abstract class HorizontalAbstract extends Combinator {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/HorizontalAbstract.java,v $ $revision$ $Date: 1998/11/02 15:54:40 $ Copyright 1998 INRIA.";

    protected HorizontalAbstract(MeasureInterface anInterword) {
	_interWord = anInterword;
    }

    public Object clone() throws CloneNotSupportedException {
	final HorizontalAbstract theResult = (HorizontalAbstract) super.clone();
	theResult._interWord = (MeasureInterface)_interWord.clone();
	return theResult;
    }

    public final MeasureInterface getInterWord() {
	return _interWord;
    }

    public final void setInterWord(MeasureInterface anInterword) {
	_interWord = anInterword;
    }

    public final void setInterWord(int anInterword) {
	setInterWord(new Measure(anInterword,UnitInterface.BLANK_TENTH));
    }
  
    protected final void preFormat2D(ContextInterface aContext,
				     FontMetricInterface aFontMetric) {
	_interWord.setFontMetric(aContext,aFontMetric);
    }

    protected final Dimension computeSize(ContextInterface aContext,
					  StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
					
					  StrategySetOrigin aSetOriginStrategy,
					  StrategyComputeSizeTill aComputeSizeTillStrategy) {
	if ( numberOfChildren() == 0 ) {
	    return new Size();
	}
	Point theFatherPoint = aContext.newPoint();
	Point thePoint = aContext.newPoint();
	Size theResult = new Size();
	GlyphInterface theCurrentSon = null;
	Dimension theCurrentSonSize;
	Dimension thePreviousSonSize = null;
	boolean theFirstFlag = true;
	PositionToFather thePositionToFather = aContext.newPositionToFather();
	PositionToBrother thePositionToBrother = null;
	int theRank = 0;
	final int theInterWordInPixel = _interWord.getValueInPixel();
	while ( aComputeSizeTillStrategy.doAlgorithm(this,theRank) ) {
	    theCurrentSon = getChild(theRank);
	    theCurrentSonSize = aComputeSizeTillStrategy.getExactOrEstimatedSize(theCurrentSon,theRank);
	    if ( theFirstFlag ) {
		thePositionToFather.movePointToPosition(thePoint,theFatherPoint);
		thePositionToBrother = aContext.newPositionToBrother(thePositionToFather);
		theFirstFlag = false;
	    }
	    else {
		thePositionToBrother.setDx(thePreviousSonSize.getExitPointX() + theInterWordInPixel);
		thePositionToBrother.setDy(thePreviousSonSize.getExitPointY());
		thePositionToBrother.movePointToPosition(thePoint);
	    }
	    aSetOriginStrategy.doAlgorithm(theCurrentSon,thePoint.getX(),thePoint.getY());
	    aComputeSizeTillStrategy.setFormatStatus(theCurrentSon,theRank);
	    theResult.growToIncludeSizeAt(theCurrentSonSize,thePoint);
	    thePreviousSonSize = theCurrentSonSize;
	    theRank++;
	}
	if ( thePreviousSonSize != null ) {
	    theResult.setExitPointX(thePoint.getX() + 
				    thePreviousSonSize.getExitPointX());
	    theResult.setExitPointY(thePoint.getY() + 
				    thePreviousSonSize.getExitPointY());
	}
	aContext.disposePoint(thePoint);
	aContext.disposePoint(theFatherPoint);
	aContext.disposePositionToFather(thePositionToFather);
	if ( thePositionToBrother != null ) {
	    aContext.disposePositionToBrother(thePositionToBrother);
	}
	return theResult;
    }
  
    public final int computeDecalage2D(GlyphInterface aGlyph,
				       DecalageInterface aDecalage) {
	return ( aGlyph.getSize().getExitPointX() + _interWord.getValueInPixel() + aDecalage.getValue() );
    }

    public final int computeInitialDecalage2D(DecalageInterface aDecalage) {
	return 0;
    }
  
    public final int computeDistance2D(DistanceInterface aDistance,
				       DecalageInterface aDecalage) {
	return ( aDistance.getValue() - aDecalage.getValue() );
    }

    public void tryAlternativeForDecalage(DecalageInterface aDecalage,
					  AlternativeInterface anAlternative) {
	anAlternative.tryBetter(ActionShrink.INSTANCE,this,aDecalage);
	DecalageInterface theDecalage = aDecalage;
	while ( theDecalage.hasBrother() ) {
	    anAlternative.tryBetter(ActionShrinkSon.INSTANCE,
				    theDecalage.getBrother(),
				    theDecalage);
	    theDecalage = theDecalage.getDecalage();
	}
    }

    protected boolean sonModified(GlyphInterface aGlyph,
				  SonModificationStatus aStatus,
				  ContextInterface aContext) {
	final int theRank = indexOf(aGlyph);
	final boolean theNotCleanFlag = ( aStatus != SonModificationStatus.KILL );
	ModificationInterface theModification = getModification();
	if ( theModification.isSonModified() ) {
	    final int thePreviousRank = theModification.getSonModifiedRank();
	    if ( thePreviousRank == Modification.NO_RANK_SPECIFIED ) {
		theModification.setSonModifiedRank(theRank);
	    }
	    else {
		theModification.setSonModifiedRank(Math.min(theRank,thePreviousRank));
	    }
	}
	else {
	    if ( ! theModification.isModified() ) {
		theModification = aContext.newModification(ModificationStatus.SON);
		setModification(theModification);
	    }
	    theModification.setSonModifiedRank(theRank);
	}
	if ( ! theNotCleanFlag ) {
	    theModification.setSonClean(true);
	}
	return true;
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("Horiz<");
	theResult.append("interword:").append(_interWord)
	    .append(";children:").append(numberOfChildren())
	    .append(super.toString()).append(">");
	return theResult.toString();
    }

    protected String getName() {
	return "H";
    }

    protected final void writeIntrinsicState(Writer anOutput) throws IOException {
	super.writeIntrinsicState(anOutput);
	anOutput.write(" Interword=\"");
	anOutput.write(Integer.toString(_interWord.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_interWord.getUnit());
    }

    private MeasureInterface _interWord;

}
