//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.PathInterface;
import figue.path.SelectionPathInterface;
import figue.resource.FontMetricInterface;
import figue.selection.HighlighterInterface;
import figue.selection.InactiveHighlighter;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Size;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.SonModificationStatus;
import figue.format.ModificationInterface;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategyComputeSizeTill;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategySetSize;
import figue.format.StopFormattingSonsException;
import figue.format.ComputedValueInterface;
import figue.format.ActionNewLine;
import figue.error.ErrorHandlerInterface;
import figue.error.Warning;
import figue.error.Error;
import figue.error.ShouldNotBeCalledError;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;
import figue.internal.BooleanWrapper;
import figue.internal.Direction;
import figue.internal.PairGlyphPath;
import figue.FacadeInterface;
import figue.InteractiveFacade;
import figue.internal.Cursor;

import java.awt.event.MouseEvent;
import java.io.IOException;
import java.io.Writer;

/**
 * [PENDING: update the comments] NullGlyph.
 * @level internal
 */

public final class NullGlyph extends GlyphInterface implements ErrorHandlerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/NullGlyph.java,v $ $revision$ $Date: 1998/11/02 15:54:42 $ Copyright 1998 INRIA.";

    private NullGlyph() {
	super();
    }

    public Object clone() throws CloneNotSupportedException {
	return this;
    }

    public GlyphInterface getParent() {
	return null;
    }

    protected void setParent(GlyphInterface aGlyph) {}

    protected void clearParent() {}
  
    public GlyphInterface getChild(int aRank) {
	return null;
    }

    public void addChild(GlyphInterface aGlyph) {}

    public void insertChild(int aRank, GlyphInterface aGlyph) {}

    public void replaceChild(int aRank, GlyphInterface aGlyph) {}

    public void removeChild() {}

    public void removeChild(int aRank) {}
  
    public int indexOf(GlyphInterface aGlyph) {
	return -1;
    }

    protected PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath) {
	return null;
    }

    public int numberOfChildren() {
	return 0;
    }

    public void notifyChange() {}
  
    public void notifyEagerChange() {}

    public ComputedValueInterface getDependent() {
	return null;
    }
  
    public void addDependent(ComputedValueInterface aValue) {}
  
    public void removeDependent() {}

    public void reportError(Error anError) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void handleError(Error anError) {
	reportError(new ShouldNotBeCalledError());
    }

    public void manageFatalError(Error anError) {
	reportError(new ShouldNotBeCalledError());
    }
  
    public void manageWarning(Warning aWarning) {
	reportError(new ShouldNotBeCalledError());
    }

    public boolean hasNextErrorHandler() {
	return false;
    }
  
    public ErrorHandlerInterface getNextErrorHandler() {
	return null;
    }

    public boolean canHandleError() {
	return false;
    }

    public int getOriginX() {
	return 0;
    }
  
    public int getOriginY() {
	return 0;
    }
  
    public void setOrigin(int x, int y) {}
  
    public void resetOrigin(int x, int y, ContextInterface aContext) {}

    protected Dimension getSize() {
	return _size;
    }
  
    protected Dimension getEagerSize() {
	return _size;
    }

    public Dimension getSizeValue() {
	return _size;
    }
  
    public void setSize(Dimension aSize) {}
  
    public void resetSize(Dimension aSize, ContextInterface aContext) {}

    public void update() {}
  
    public void updateEager() {}

    protected  Dimension computeSize(ContextInterface aContext,
				     StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
				   
				     StrategySetOrigin aSetOriginStrategy,
				     StrategyComputeSizeTill aComputeSizeTillStrategy) {
	return null;
    }

    public void format2D(ContextInterface aContext,
			 FontMetricInterface aFontMetric,
			 AlternativeInterface anAlternative,
			 DistanceInterface aDistance,
			 StrategyReadyToFormat aReadyToFormatStrategy,
			 StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			 StrategySetOrigin aSetOriginStrategy,
			 StrategySetSize aSetSizeStrategy,
			 boolean aMustFormatFlag)
	throws StopFormattingSonsException {}
    
    public final boolean redispatchEvent(InteractiveFacade aFacade,int aMouseEventType,MouseEvent anEvent) {
	return false;
    }

    public int computeDistance2D(DistanceInterface aDistance,
				 DecalageInterface aDecalage) {
	return 0;
    }
  
    public int computeDecalage2D(GlyphInterface aGlyph,
				 DecalageInterface aDecalage) {
	return 0;
    }

    public int computeInitialDecalage2D(DecalageInterface aDecalage) {
	return 0;
    }

    public void tryAlternative(AlternativeInterface anAlternative) {}
  
    public void tryAlternativeForInitial(AlternativeInterface anAlternative) {}
  
    public void tryAlternativeForDistance(DistanceInterface aDistance,
					  AlternativeInterface anAlternative) {}
  
    public void tryAlternativeForDecalage(DecalageInterface aDecalage,
					  AlternativeInterface anAlternative) {}
  
    public boolean isClean() {
	return true;
    }
  
    public boolean isDirty() {
	return false;
    }

    public boolean isNew() {
	return false;
    }
  
    public boolean isProceed() {
	return false;
    }
  
    public boolean isUnstable() {
	return false;
    }
  
    public void setClean() {}
  
    public void setProceed() {}
  
    public void setDirty() {}

    public void setNew() {}
  
    public void setUnstable() {}

    public void markAllBranchDirty(SonModificationStatus aStatus,
				   ContextInterface aContext) {}

    public void cleanModificationAndUpdateRefreshZone(Rectangle aVisibleZone,
						      Rectangle aRefreshZone,
						      BooleanWrapper aRefreshZoneNotInitializedFlag,
						      Rectangle aRectangle,
						      ContextInterface aContext) {}

    protected void cleanModification(ContextInterface aContext) {}

    public PairGlyphPath getDeepestContains(int x,
					    int y, 
					    Rectangle aBoundingBox,
					    ContextInterface aContext) {
    
	return null;
    }

    public Cursor getCursor() {
	return null;
    }
  
    protected void setCursor(Cursor aCursor) {}
  
    protected boolean isFinalTarget() {
	return false;
    }

    protected void refresh() {}

    public boolean draw2D(ContextInterface aContext,
			  Transformer aTransformer,
			  SelectionPathInterface aSelectionPath,
			  Rectangle aRefreshZone,
			  Rectangle aBoundingBox) { return true; }

    protected void draw2DClean(ContextInterface aContext,
			       Transformer aTransformer,
			       SelectionPathInterface aSelectionPath,
			       Rectangle aRefreshZone,
			       Rectangle aBoundingBox) {}

    protected boolean draw2D(ContextInterface aContext,
			     Transformer aTransformer,
			     Rectangle aRefreshZone,
			     Rectangle aBoundingBox) { return true; }

    protected void draw2DClean(ContextInterface aContext,
			       Transformer aTransformer,
			       Rectangle aRefreshZone,
			       Rectangle aBoundingBox) {}

    protected ContextMementoInterface draw2DSelections(ContextInterface aContext,
						       Transformer aTransformer,
						       SelectionPathInterface aSelectionPath) {
	return null;
    }

    public HighlighterInterface getHighlighter(SelectionPathInterface aSelectionPath) {
	return InactiveHighlighter.INSTANCE;
    }

    protected SelectionPathInterface zoomInSelection(SelectionPathInterface aSelectionPath,
						     int aRank) {
	return null;
    }

    public FacadeInterface getFacade() {
	return null;
    }
    
    protected boolean proceedFurtherAndCheckAwake(Size aCurrentSize) {
	return false;
    }

    public boolean improveEstimatedHeightAndCheckAwake(int anHeight) {
	return false;
    }

    public void improveSizeEstimationAndUpdateParents(ContextInterface aContext,
						      int aRank) {}

    protected StrategyImproveSizeEstimation chooseAlgorithm(StrategyImproveSizeEstimation aStrategy) {
	return StrategyImproveSizeEstimation.INACTIVE;
    }
  
    protected void setSizeEstimation(Dimension aCurrentSize) {}
  
    public Dimension getSizeEstimation() {
	return _size;
    }
    
    protected void computeBoundingBox(Rectangle aRectangle,
				      Transformer aTransformer) {}
  
    public void computeBoundingBox(Rectangle aRectangle) {}
    
    public void computeOldBoundingBox(Rectangle aRectangle,
				      int anOldX,
				      int anOldY,
				      Dimension anOldSize) {}
    
    protected void computeEstimatedBoundingBox(Rectangle aRectangle,
					       Transformer aTransformer) {}
  
    protected void computeEstimatedBoundingBox(Rectangle aRectangle) {}
    
    protected Direction checkRefreshZone(Rectangle aRefreshZone,
					 Rectangle aBoundingBox) {
	return null;
    }
    
    protected Direction checkRefreshZoneAgainstEstimation(Rectangle aRefreshZone,
							  Rectangle aBoundingBox) {
	return null;
    }

    protected void doTranslate(Rectangle aRectangle) {}
  
    protected void undoTranslate(Rectangle aRectangle) {}

    public void doTranslatesZones(Rectangle aRefreshZone,
				  Rectangle aVisibleZone,
				  int aRank) {}
    public void undoTranslateZones(Rectangle aRefreshZone,
				   Rectangle aVisibleZone,
				   int aRank) {}
    
    public boolean isActionNoneComplient() {
	return false;
    }
  
    public boolean isActionNewLineComplient(ActionNewLine anAction) {
	return false;
    }
  
    public void executeActionNewLine(GlyphInterface aGlyph) 
	throws StopFormattingSonsException {}
  
    public boolean isActionShrinkComplient() {
	return false;
    }

    public  void executeActionShrink() throws StopFormattingSonsException {}

    public boolean isActionShrinkSonComplient() {
	return false;
    }

    public void executeActionShrinkSon() {}

    protected ModificationInterface getModification() {
	return null;
    }
  
    protected void setModification(ModificationInterface aModification) {}

    protected boolean sonModified(GlyphInterface aGlyph,
				  SonModificationStatus aStatus,
				  ContextInterface aContext) {
	return false;
    }

    public void computeCuttingCosts(AlternativeInterface anAlternative,
				    Rectangle aRectangle,
				    Transformer aTransformer) {}
    
    public final void writeAsXML(Writer anOutput) throws IOException {
	writeState(anOutput);
    }
    public final void writeState(Writer anOutput) throws IOException {
	anOutput.write("<NullGlyph/>");
    }
    public final void writeIntrinsicState(Writer anOutput) throws IOException {
    }
    public final boolean writeExtrinsicState(Writer anOutput) throws IOException {
	return false;
    }
    protected final String getName() {
	return "Null";
    }

    private final Dimension _size = new Dimension();

    final static NullGlyph INSTANCE = new NullGlyph();

}
