//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.geometry.Transformer;
import figue.geometry.Dimension;
import figue.geometry.Rectangle;
import figue.path.LinearPath;
import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.resource.FontMetricInterface;
import figue.format.AlternativeInterface;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategySetSize;
import figue.format.StrategyComputeSizeTill;
import figue.format.StrategyReadyToFormat;
import figue.format.StopFormattingSonsException;
import figue.format.SonModificationStatus;
import figue.error.ShouldNotBeCalledError;
import figue.FacadeInterface;
import figue.internal.PairGlyphPath;
import figue.internal.ContextInterface;

/**
 * [PENDING: update the comments] RootGlyph.
 * @level internal
 */

public final class RootGlyph extends CompositeGlyphArity1 {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/RootGlyph.java,v $ $revision$ $Date: 1998/11/02 15:54:43 $ Copyright 1998 INRIA.";
	
    public RootGlyph(FacadeInterface aFacade) { 
	super();
	_facade = aFacade;
	_transformerCache = new Transformer();
    }
	
    protected final void computeBoundingBox(Rectangle aRectangle,
					    Transformer aTransformer) {
	super.computeBoundingBox(aRectangle,aTransformer);
	final Dimension theSize = getSizeValue();
	aRectangle.translate(theSize.getUpperLeftX(),theSize.getUpperLeftY());
    }
	
    public final void computeBoundingBox(Rectangle aRectangle) {
	super.computeBoundingBox(aRectangle);
	final Dimension theSize = getSizeValue();
	aRectangle.translate(theSize.getUpperLeftX(),theSize.getUpperLeftY());
    }
	
    public final void computeOldBoundingBox(Rectangle aRectangle,
					    int anOldX,
					    int anOldY,
					    Dimension anOldSize) {
	super.computeOldBoundingBox(aRectangle,anOldX,anOldY,anOldSize);
	aRectangle.translate(anOldSize.getUpperLeftX(),anOldSize.getUpperLeftY());
    }

    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	if ( numberOfChildren() == 1 ) {
	    GlyphInterface theSon = getChild(0);
	    aReadyToFormatStrategy.doAlgorithm(aContext,theSon);
	    if ( ! theSon.isClean() ) {
		theSon.format2D(aContext,aFontMetric,anAlternative,aDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
		theSon.setClean();
		aSetSizeStrategy.doAlgorithm(this,theSon.getSizeValue());
	    }
	}
	else {
	    aSetSizeStrategy.doAlgorithm(this,new Dimension());
	}
	setClean();
    }

    public final void update() {
	reportError(new ShouldNotBeCalledError());
    }

    public final void updateEager() {
	reportError(new ShouldNotBeCalledError());
    }

    protected Dimension computeSize(ContextInterface aContext,
				    StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
					
				    StrategySetOrigin aSetOriginStrategy,
				    StrategyComputeSizeTill aComputeSizeTillStrategy) {
	if ( numberOfChildren() != 0 ) {
	    return aComputeSizeTillStrategy.getExactOrEstimatedSize(getChild(0),0);
	}
	else {
	    return new Dimension();
	}
    }

    public final int computeDecalage2D(GlyphInterface aGlyph,
				       DecalageInterface aDecalage) {
	reportError(new ShouldNotBeCalledError());
	return -1;
    }

    public final int computeInitialDecalage2D(DecalageInterface aDecalage) {
	reportError(new ShouldNotBeCalledError());
	return -1;
    }

    public final int computeDistance2D(DistanceInterface aDistance,
				       DecalageInterface aDecalage) {
	reportError(new ShouldNotBeCalledError());
	return -1;
    }
	
    public final void tryAlternative(AlternativeInterface anAlternative) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void tryAlternativeForInitial(AlternativeInterface anAlternative) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void tryAlternativeForDistance(DistanceInterface aDistance,
						AlternativeInterface anAlternative) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void tryAlternativeForDecalage(DecalageInterface aDecalage,
						AlternativeInterface anAlternative) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void improveSizeEstimationAndUpdateParents(ContextInterface aContext,
							    int aRank) {
	if ( ! isProceed() ) {
	    setProceed();
	}
	StrategyComputeSizeTill.THE_RANK.setRank(0);
	setSizeEstimation(computeSize(aContext,
				      StrategyImproveSizeEstimation.INACTIVE,
				      StrategySetOrigin.NO_INCREMENTAL,
				      StrategyComputeSizeTill.THE_RANK));
	aContext.majorFormatAdvance(); /// +++
    }

    public final void setOrigin(int x, int y) {
	super.setOrigin(x,y);
	getChild(0).setOrigin(x,y);
    }

    public final void resetOrigin(int x, int y, ContextInterface aContext) {
	super.resetOrigin(x,y,aContext);
	getChild(0).resetOrigin(x,y,aContext);
    }

    protected final void doTranslate(Rectangle aRectangle) {
	if ( numberOfChildren() == 1 ) {
	    final Dimension theSonSize = getChild(0).getSizeValue();
	    aRectangle.translate(- theSonSize.getUpperLeftX(),- theSonSize.getUpperLeftY());
	}
    }

    protected final void undoTranslate(Rectangle aRectangle) {
	if ( numberOfChildren() == 1 ) {
	    final Dimension theSonSize = getChild(0).getSizeValue();
	    aRectangle.translate(theSonSize.getUpperLeftX(),theSonSize.getUpperLeftY());
	}
    }

    public final void doTranslatesZones(Rectangle aRefreshZone,
					Rectangle aVisibleZone,
					int aRank) {
	if ( numberOfChildren() == 1 ) {
	    final Dimension theSonSize = getChild(0).getSizeValue();
	    final int theUpperLeftX = - theSonSize.getUpperLeftX();
	    final int theUpperLeftY = - theSonSize.getUpperLeftY();
	    aRefreshZone.translate(theUpperLeftX,theUpperLeftY);
	    aVisibleZone.translate(theUpperLeftX,theUpperLeftY);   
	}
    }
    public final void undoTranslateZones(Rectangle aRefreshZone,
					 Rectangle aVisibleZone,
					 int aRank) {
	if ( numberOfChildren() == 1 ) {
	    final GlyphInterface theChild = getChild(0);
	    final Dimension theSonSize = theChild.getSizeValue();
	    if ( theSonSize != null ) {
		final int theUpperLeftX = theSonSize.getUpperLeftX();
		final int theUpperLeftY = theSonSize.getUpperLeftY();
		aRefreshZone.translate(theUpperLeftX,theUpperLeftY);
		aVisibleZone.translate(theUpperLeftX,theUpperLeftY);
	    }
	}
    }

    public final PairGlyphPath getDeepestContains(int x, 
						  int y, 
						  Rectangle aBoundingBox,
						  ContextInterface aContext) { 
	if ( numberOfChildren() == 0 ) {
	    return null;
	}
	GlyphInterface theSon = getChild(0);
	Dimension theSonSize = theSon.getSizeValue();
	if ( theSonSize == null ) {
	    return null;
	}
	PairGlyphPath theResult 
	    = theSon.getDeepestContains(x - theSonSize.getUpperLeftX(),
					y - theSonSize.getUpperLeftY(),
					aBoundingBox,
					aContext);
	if ( theResult != null ) {
	    PathInterface theExtendedPath =  new LinearPath();
	    if ( theResult.getPath() != null ) {
		theExtendedPath.addChild(theResult.getPath());
	    }
	    theResult.setPath(theExtendedPath);
	}
	return theResult;
    }
	
    public final boolean draw2D(ContextInterface aContext, 
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	final boolean theIsCleanFlag;
	if ( numberOfChildren() == 1 ) {
	    if ( isClean() ) {
		theIsCleanFlag = true;
	    }
	    else {
		if ( isProceed() ) {
		    theIsCleanFlag = false;
		}
		else {
		    return false;
		}
	    }
	    GlyphInterface theSon = getChild(0);
	    Dimension theSonSize = theSon.getSizeEstimation(); 
	    _transformerCache.setLocation(theSonSize.getUpperLeftX(),
					  theSonSize.getUpperLeftY());
	    aRefreshZone.translate(- theSonSize.getUpperLeftX(),
				   - theSonSize.getUpperLeftY());
	    SelectionPathInterface theSelectionSon = downInSelection(aSelectionPath);
	    if ( theSelectionSon != null ) {
		if ( theIsCleanFlag ) {
		    theSon.draw2DClean(aContext,_transformerCache,theSelectionSon,aRefreshZone,aBoundingBox);
		}
		else {
		    theSon.draw2D(aContext,_transformerCache,theSelectionSon,aRefreshZone,aBoundingBox);
		}
	    }
	    else {
		if ( theIsCleanFlag ) {
		    theSon.draw2DClean(aContext,_transformerCache,aRefreshZone,aBoundingBox);
		}
		else {
		    theSon.draw2D(aContext,_transformerCache,aRefreshZone,aBoundingBox);
		}
	    }
	}
	else {
	    theIsCleanFlag = true;
	    aContext.fillRectangle(0,0,aRefreshZone.getWidth(),aRefreshZone.getHeight());
	}
	aContext.flush();
	return theIsCleanFlag;
    }

    protected final void draw2DClean(ContextInterface aContext, 
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	reportError(new ShouldNotBeCalledError());
    }

    protected final boolean draw2D(ContextInterface aContext, 
				   Transformer aTransformer,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox) {
	reportError(new ShouldNotBeCalledError());
	return false;
    }
    
    protected final void draw2DClean(ContextInterface aContext, 
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	reportError(new ShouldNotBeCalledError());
    }

    public final FacadeInterface getFacade() {
	return _facade;
    }

    protected final boolean sonModified(GlyphInterface aGlyph,
					SonModificationStatus aStatus,
					ContextInterface aContext) {
	return true;
    }
	
    protected final PathInterface computePathTo(GlyphInterface aChild,PathInterface aPath) {
	final PathInterface theResult = new LinearPath();
	theResult.addChild(new LinearPath(indexOf(aChild)));
	if ( aPath != null ) {
	    theResult.addChild(aPath);
	}
	return theResult;
    }

    public final void computeCuttingCosts(AlternativeInterface anAlternative,
					  Rectangle aRectangle,
					  Transformer aTransformer) {
	if ( numberOfChildren() == 1 ) {
	    GlyphInterface theSon = getChild(0);
	    Dimension theSonSize = theSon.getSizeValue();
	    aTransformer.setLocation(theSonSize.getUpperLeftX(),
				     theSonSize.getUpperLeftY());
	    theSon.computeCuttingCosts(anAlternative,aRectangle,aTransformer);
	}
    }

    public final void dispose() {
	_facade = null;
	_transformerCache = null;
    }

    public String toString() {
	StringBuffer theResult = new StringBuffer("RootGlyph<");
	theResult.append(super.toString()).append(">");
	return theResult.toString();
    }

    protected String getName() {
	return "Root";
    }

    private FacadeInterface _facade;
    private Transformer _transformerCache;

}

