//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.SelectionPathInterface;
import figue.resource.PortableFontInterface;
import figue.resource.FontMetricInterface;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.AlternativeInterface;
import figue.format.ModificationInterface;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategySetSize;
import figue.format.StopFormattingSonsException;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;

/**
 * [PENDING: update the comments] Symbol.
 */

public final class Symbol extends AtomAbstract {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Symbol.java,v $ $revision$ $Date: 1998/11/02 15:54:43 $ Copyright 1998 INRIA.";
  
    public Symbol(String aString, PortableFontInterface aFont) {
	super();
	_text = aString;
	_size = null;
	_font = aFont;
    }

    protected final String getText() { 
	return _text; 
    }

    public final Dimension getSizeValue() {
	return _size;
    }

    public final void setSize(Dimension aSize) {
	_size = aSize;
    }

    public final void resetSize(Dimension aSize, ContextInterface aContext) {
	if ( ( _size != null ) && ( ! _size.sameShape(aSize) ) ) {
	    ModificationInterface theModification = getModification();
	    if ( ! theModification.isModified() ) {
		theModification = aContext.newModification();
		setModification(theModification);
	    }
	    theModification.setSize(getSizeValue());
	}
	_size = aSize;
    }
  
    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	final FontMetricInterface theFontMetric = _font.getFontMetric();
	aSetSizeStrategy.doAlgorithm(this,
				     new Dimension(theFontMetric.getAscent(),
						   theFontMetric.stringWidth(_text),
						   theFontMetric.getDescent()));
	aDistance.checkAfter(this,anAlternative);
    }
    
    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	ContextMementoInterface theMemento = null;
	theMemento = draw2DSelections(aContext,aTransformer,aSelectionPath);
	aContext.drawSymbol(_font,
			    _text,
			    aTransformer.applyToX(getOriginX()),
			    aTransformer.applyToY(getOriginY()));
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
    }

    protected final boolean draw2D(ContextInterface aContext,
				   Transformer aTransformer,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	aContext.drawSymbol(_font,
			    _text,
			   aTransformer.applyToX(getOriginX()),
			   aTransformer.applyToY(getOriginY()));
    }

    private String _text;
    private PortableFontInterface _font;
    private Dimension _size;
  
}
