package figue.custom;

import figue.resource.FontFamily;
import figue.error.ImplementationError;

public abstract class ConfigurationInterface {

    public abstract java.awt.Font newAwtFont(FontFamily aFamily,
					     int aStyle,
					     int aPointSize);
    
    public final static ConfigurationInterface INSTANCE = checkConfiguration();

    private final static ConfigurationInterface checkConfiguration() {
	final int theJavaVersion = getJavaVersion();
	final int theOperatingSystem = getOperatingSystem();
	final int theVirtualMachineVendor = getVirtualMachineVendor();
	String theClassToLoad = "figue.custom.Default";
	if ( ( theVirtualMachineVendor == VENDOR_MICROSOFT ) && 
	     ( theOperatingSystem == SYSTEM_NT) ) {
	    theClassToLoad = "figue.custom.Microsoft";
	}
	try {
	    final Class theClass = Class.forName(theClassToLoad);
	    return (ConfigurationInterface)(theClass.newInstance());
	}
	catch ( ClassNotFoundException anException ) {
	    anException.printStackTrace();
	    throw new ImplementationError("Unexpected");
	}
	catch ( SecurityException anException ) {
	    throw new ImplementationError("Unexpected");
	}
	catch ( InstantiationException anException ) {
	    throw new ImplementationError("Unexpected");
	}
	catch ( IllegalAccessException anException ) {
	    throw new ImplementationError("Unexpected");
	}
    }

    private static final int getJavaVersion() {
	int theVersion = JAVA_UNKNOWN;
	try {
	    Class.forName("java.awt.AWTEvent");
	    theVersion = JAVA_1_1;
	}
	catch ( ClassNotFoundException anException ) {
	    ;
	}
	catch ( SecurityException anException ) {
	    ;
	}
	return theVersion;
    }

    private static final int getOperatingSystem() {
	final String theSystem = System.getProperty("os.name");
	if ( theSystem.equals("Windows NT") ) {
	    return SYSTEM_NT;
	}
	else {
	    if ( theSystem.equals("Solaris") ) {
		return SYSTEM_SOLARIS;
	    }
	    else {
		if ( theSystem.equals("Linux") ) {
		    return SYSTEM_LINUX;
		}
		else {
		    return SYSTEM_UNKNOWN;
		}
	    } 
	}
    }

    private final static int getVirtualMachineVendor() {
	final String theVendor = System.getProperty("java.vendor.url");
	if ( theVendor.equals("http://www.microsoft.com/") ) {
	    return VENDOR_MICROSOFT;
	}
	else {
	    return VENDOR_UNKNOWN;
	}
    }

    private final static int JAVA_UNKNOWN = 0;
    private final static int JAVA_1_1 = JAVA_UNKNOWN + 1;
    
    private final static int SYSTEM_UNKNOWN = 0;
    private final static int SYSTEM_NT = SYSTEM_UNKNOWN + 1;
    private final static int SYSTEM_SOLARIS = SYSTEM_NT + 1;
    private final static int SYSTEM_LINUX = SYSTEM_SOLARIS + 1;
    
    private final static int VENDOR_UNKNOWN = 0;
    private final static int VENDOR_MICROSOFT = VENDOR_UNKNOWN + 1;

}
