//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.doclet;

import com.sun.javadoc.*;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.io.FileWriter;
import java.io.LineNumberReader;
import java.io.FileReader;
import java.util.Hashtable;

public abstract class HtmlWriter {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/doclet/HtmlWriter.java,v $ $revision$ $Date: 1998/11/02 15:54:48 $ Copyright 1998 INRIA.";

    protected HtmlWriter(String anOutputDirectory,
			 String aTemplateDirectory,
			 String aTemplateFile,
			 Hashtable aKnownTypeList) {
	_output = anOutputDirectory;
	_template = aTemplateDirectory;
	_templateFile = aTemplateFile;
	_knownTypes = aKnownTypeList;
	_templateInputStream = null;
	_outputStream = null;
    }

    public final void go() throws IOException {
	start();
	document();
	end();
    }

    protected abstract void document() throws IOException;

    private final void start() throws IOException {
	_outputStream = new FileWriter(_output + "/" + getOutputFileName());
    }

    private final void end() throws IOException {
	_outputStream.close();
	_outputStream = null;
    }

    protected void title(String aTitle, Writer aStream) throws IOException {
	aStream.write("<H1>" + aTitle + "</H1>\n</TD></TR>\n");
    }

    protected void section(Writer aStream, String aTitle, String aTag) throws IOException {
	aStream.write("<TR><TD></TD>\n");
	aStream.write("<TD>\n");
	if ( aTag != null ) {
	    aStream.write("<A NAME=\"" + aTag.toUpperCase() + "\"></A>\n");
	}
	aStream.write("<H3><IMG SRC=\"../../images/box.gif\" ALT=\"*\"> " + aTitle + "</H3>\n");
    }

    protected void endSection(Writer aStream) throws IOException {
	aStream.write("</TD></TR>\n");
    }

    protected boolean isKnownType(Type aClass) {
	if ( _knownTypes.containsKey(aClass) ) {
	    return ((InfoShunk)(_knownTypes.get(aClass)))._knownFlag;
	}
	else {
	    return false;
	}
    }

    protected void linkToPackage(Writer aStream,PackageDoc aPackage) throws IOException {
	final String theName = aPackage.name();
	aStream.write("<A HREF=\"Package-" + 
		      theName + ".html\">" + theName + "</A>");
    }

    protected void linkToStructure(Writer aStream, ClassDoc aClass) throws IOException {
	if ( isKnownType(aClass) ) {
	    aStream.write("<A HREF=\"" + aClass.qualifiedName() + ".html\">" 
			  + aClass.name() + "</A>");
	}
	else {
	    aStream.write(aClass.name());
	}
    }

    protected int linkToType(Writer aStream, Type aType) throws IOException {
	final String theName = aType.typeName();
	if ( isKnownType(aType) ) {
	    aStream.write("<A HREF=\"" + aType.qualifiedTypeName() 
			  + ".html\">" + theName + "</A>");
	}
	else {
	    aStream.write(theName);
	}
	return theName.length();
    }

    protected void linkToMember(Writer aStream, MemberDoc aMember) throws IOException {
	final ClassDoc theClass = aMember.containingClass();
	if ( isKnownType(theClass) ) {
	    if ( aMember.isField() ) {
		final FieldDoc theField = (FieldDoc)aMember;
		final String theName = theField.name();
		aStream.write("<A HREF=\"" + theClass.qualifiedName() + "#" +
			      theName+ ".html\">" + theName + "</A>");
	    }
	    else {
		final ExecutableMemberDoc theMethod = (ExecutableMemberDoc)aMember;
		final String theName = theMethod.name() ;
		aStream.write("<A HREF=\"" + theClass.qualifiedName() + ".html#" +
			      theName + theMethod.signature()
			      + "\">" + 
			      theName + theMethod.flatSignature() + "</A>");
	    }
	}
	else {
	    aStream.write(aMember.qualifiedName());
	}
    }

    protected void highlightName(Writer aStream, String aName) throws IOException {
	aStream.write("<STRONG><FONT SIZE=\"+1\">" + aName + 
		      "</FONT></STRONG><BR>\n<BR>\n");
    }

    protected void comments(Writer aStream, Doc anObject) throws IOException {
	aStream.write("<P>" + anObject.commentText() + "</P>\n");
    }

    protected String shortComment(Doc anObject) {
	final String theComment = anObject.commentText();
	if ( theComment != null ) {
	    final int theRank = theComment.indexOf('.');
	    if ( theRank != -1 ) {
		return theComment.substring(0,theRank + 1);
	    }
	    else {
		return theComment;
	    }
	}
	return "";
    }

    protected boolean seeTags(Writer aStream, Doc anElement) throws IOException {
	SeeTag[] theTags = anElement.seeTags();
	final int theSize = theTags.length;
	if ( theSize > 0 ) {
	    aStream.write("<DL><DT><STRONG>See also:</STRONG><DD>");
	    SeeTag theCurrentTag;
	    for ( int theRank = 0; theRank < theSize; theRank++ ) {
		theCurrentTag = theTags[theRank];
		if ( theRank > 0 ) {
		    aStream.write(", ");
		}
		if ( theCurrentTag.referencedClass() == null ) {
		    aStream.write(theCurrentTag.referencedClassName());
		    if ( theCurrentTag.referencedMemberName() != null ) {
			aStream.write("." + theCurrentTag.referencedMemberName());
		    }
		}
		else {
		    if ( theCurrentTag.referencedMember() == null ) {
			linkToType(aStream,theCurrentTag.referencedClass());
		    }
		    else {
			linkToMember(aStream,theCurrentTag.referencedMember());
		    }
		}
	    }
	    aStream.write("</DL>\n");
	    return true;
	}
	else {
	    return false;
	}
    }

    protected int modifiers(Writer aStream, ClassDoc aClass) throws IOException {
	int theResult = modifiers(aStream,(ProgramElementDoc)aClass);
	if ( aClass.isAbstract() ) {
	    aStream.write("abstract ");
	    theResult += "abstract ".length();
	}
	return theResult;
    }

    protected int modifiers(Writer aStream, ProgramElementDoc anElement) throws IOException {
	int theLength = 0;
	if ( anElement.isPublic() ) {
	    aStream.write("public ");
	    theLength += "public ".length();
	}
	else {
	    if ( anElement.isProtected() ) {
		aStream.write("protected ");
		theLength += "protected ".length();
	    }
	    else {
		if ( anElement.isPrivate() ) {
		    aStream.write("private ");
		    theLength += "private ".length();
		}
	    }
	}
	if ( anElement.isFinal() ) {
	    aStream.write("final ");
	    theLength += "final ".length();
	}
	if ( anElement.isStatic() ) {
	    aStream.write("static ");
	    theLength += "static ".length();
	}
	return theLength;
    }

    protected void copyTemplateHeader(Writer aStream) throws IOException {
	_templateInputStream 
	    = new LineNumberReader(new FileReader(_template + _templateFile));
	String theCurrentLine = _templateInputStream.readLine();
	while ( theCurrentLine.indexOf(Constants.START_BODY) == -1 ) {
	    aStream.write(theCurrentLine);
	    aStream.write('\n');
	    theCurrentLine = _templateInputStream.readLine();
	}
    }
    
    protected void copyTemplateFooter(Writer aStream) throws IOException {
	String theCurrentLine = _templateInputStream.readLine();
	while ( theCurrentLine.indexOf(Constants.END_BODY) == -1 ) {
	    theCurrentLine = _templateInputStream.readLine();
	}
	theCurrentLine = _templateInputStream.readLine();
	while ( theCurrentLine != null  ) {
	    aStream.write(theCurrentLine);
	    aStream.write('\n');
	    theCurrentLine = _templateInputStream.readLine();
	}
	_templateInputStream.close();
	_templateInputStream = null;
    }

   protected String getInheritedMethods(ClassDoc aClass) {
	return ((InfoShunk)(_knownTypes.get(aClass)))._methodList;
    }

    protected abstract String getOutputFileName();
    
    protected Writer getOutputStream() {
	return _outputStream;
    }

    private String _output;
    private String _template;
    private String _templateFile;
    private Hashtable _knownTypes;
    private LineNumberReader _templateInputStream;
    private FileWriter _outputStream; 

}
