//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.doclet;

import com.sun.javadoc.*;
import java.util.Stack;
import java.util.Hashtable;
import java.util.Enumeration;

public final class InfoShunk {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/doclet/InfoShunk.java,v $ $revision$ $Date: 1998/11/02 15:54:48 $ Copyright 1998 INRIA.";

    public InfoShunk(ClassDoc aClass) {
	super();
	_class = aClass;
	_constructors = _class.constructors();
	_constructorSize = _constructors.length;
	_methods = _class.methods();
	_methodSize = _methods.length;
	_fields = _class.fields();
	_fieldSize = _fields.length;
	_interfaces = _class.interfaces();
	_interfaceSize = _interfaces.length;
	_inheritance = initInheritance(_class);
	_subsumers = initSubsumption(new Hashtable(),_class);
	_methodList = "";
	_knownFlag = false;
	_updatedFlag = false;
    }

    public void filter(int aLevel) {
	_knownFlag = ! Utility.filter(aLevel,_class);
	_constructorSize = _constructorSize - Utility.filter(aLevel,_constructors);
	if ( _constructorSize > 0 ) {
	    Utility.quickSort(_constructors,0,_constructorSize-1);
	}
	_methodSize = _methodSize - Utility.filter(aLevel,_methods);
	if ( _methodSize > 0 ) {
	    Utility.quickSort(_methods,0,_methodSize-1);
	}
	_fieldSize = _fieldSize - Utility.filter(aLevel,_fields);
	if ( _fieldSize > 0 ) {
	    Utility.quickSort(_fields,0,_fieldSize-1);
	}
	_interfaceSize = _interfaceSize - Utility.filter(aLevel,_interfaces);
	StringBuffer theMethodList = new StringBuffer();
	MethodDoc theMethod;
	final int theSize = _methodSize;
	final String theClassName = _class.qualifiedName();
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
		theMethod = _methods[theRank];
		if ( theRank > 0 ) {
		    theMethodList.append(", ");
		}
		theMethodList.append("<A HREF=\"" + theClassName + ".html#" +
				     theMethod.name() + theMethod.signature()
				     + "\">" + 
				     theMethod.name() + "</A>");
	}
	_methodList = theMethodList.toString();
    }

    public boolean isUpToDate() {
	return _updatedFlag;
    }

    public void update(Hashtable aDico) {
	ClassDoc theClass;
	InfoShunk theInfo;
	final Enumeration theSubsumers = _subsumers.keys();
	while ( theSubsumers.hasMoreElements() ) {
	    theClass = (ClassDoc)(theSubsumers.nextElement());
	    if ( aDico.containsKey(theClass) ) {
		theInfo = (InfoShunk)(aDico.get(theClass));
		if ( ! theInfo.isUpToDate() ) {
		    theInfo.update(aDico);
		}
	    }
	}
	if ( ! _inheritance.isEmpty() ) {
	    theClass = (ClassDoc)(_inheritance.elementAt(_inheritance.size() - 1));
	    if ( aDico.containsKey(theClass) ) {
		theInfo = (InfoShunk)(aDico.get(theClass));
		if ( ! theInfo.isUpToDate() ) {
		    theInfo.update(aDico);
		}
	    }
	}
	final int theMethodSize = _methods.length;
	MethodDoc theMethod;
	MethodDoc theInheritedMethod;
	for ( int theRank = 0; theRank < theMethodSize; theRank++ ) {
	    theMethod = _methods[theRank];
	    if ( notCommented(theMethod) ) {
		theInheritedMethod = inheritedMethod(aDico,theMethod);
		if ( theInheritedMethod != null ) {
		    _methods[theRank] = theInheritedMethod;
		}
	    }
	}
	_updatedFlag = true;
    }

    private boolean notCommented(MethodDoc aMethod) {
	if ( aMethod.tags().length > 0 ) {
	    return false;
	}
	else {
	    final String theComments = aMethod.commentText();
	    return ( ( theComments == null ) || ( theComments.length() == 0 ) );
	}
    }

    private MethodDoc inheritedMethod(Hashtable aDico,MethodDoc aMethod) {
	final int theInheritanceSize = _inheritance.size();
	ClassDoc theClass;
	MethodDoc theMethod;
	for ( int theRank = theInheritanceSize - 1; theRank >= 0; theRank-- ) {
	    theClass = (ClassDoc)(_inheritance.elementAt(theRank));
	    theMethod = findMethod(aDico,theClass,aMethod);
	    if ( ( theMethod != null ) && ( ! notCommented(theMethod) ) ) {
		return theMethod;
	    }
	}
	final Enumeration theSubsumers = _subsumers.keys();
	while ( theSubsumers.hasMoreElements() ) {
	    theClass = (ClassDoc)(theSubsumers.nextElement());
	    theMethod = findMethod(aDico,theClass,aMethod);
	    if ( ( theMethod != null ) && ( ! notCommented(theMethod) ) ) {
		return theMethod;
	    }
	}
	return null;
    }
    
    private MethodDoc findMethod(Hashtable aDico,ClassDoc aClass, MethodDoc aMethod) {
	if ( aDico.containsKey(aClass) ) {
	    final InfoShunk theInfo = (InfoShunk)(aDico.get(aClass));
	    final String theName = aMethod.name();
	    final String theSignature = aMethod.signature();
	    MethodDoc[] theMethods = theInfo._methods;
	    final int theSize = theMethods.length;
	    MethodDoc theCurrentMethod;
	    for ( int theRank = 0; theRank < theSize; theRank++ ) {
		theCurrentMethod = theMethods[theRank];
		if ( theName.equals(theCurrentMethod.name()) &&
		     theSignature.equals(theCurrentMethod.signature()) ) {
		    return theCurrentMethod;
		}
	    }
	}
	return null;
    }
    
    private static Stack initInheritance(ClassDoc aClass) {
	final Stack theResult = new Stack();
	if ( ! aClass.isInterface() ) {
	    ClassDoc theSuper = aClass;
	    while ( theSuper != null ) {
		theResult.push(theSuper);
		theSuper = theSuper.superclass();
	    }
	}
	return theResult;
    }

    private static Hashtable initSubsumption(Hashtable theSubsumers,
					     ClassDoc aClass) {
	final ClassDoc theSuperClass = aClass.superclass();
	if ( ( theSuperClass != null )  && 
	     ( ! theSubsumers.containsKey(theSuperClass)) ) {
	    if ( theSuperClass.isInterface() ) {
		theSubsumers.put(theSuperClass,"x");
	    }
	    initSubsumption(theSubsumers,theSuperClass);
	}
	final ClassDoc[] theInterfaces = aClass.interfaces();
	for ( int theRank = 0; theRank < theInterfaces.length; theRank++ ) {
	    if ( ! theSubsumers.containsKey(theInterfaces[theRank]) ) {
		theSubsumers.put(theInterfaces[theRank],"x");
		initSubsumption(theSubsumers,theInterfaces[theRank]);
	    }
	}
	return theSubsumers;
    }
    
    private ClassDoc _class;
    private ClassDoc[] _interfaces;
    private int _interfaceSize;
    ConstructorDoc[] _constructors;
    int _constructorSize;
    MethodDoc[] _methods;
    int _methodSize;
    FieldDoc[] _fields;
    int _fieldSize;
    Stack _inheritance;
    Hashtable _subsumers;
    String _methodList;
    private boolean _updatedFlag;
    boolean _knownFlag;

}
