//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.doclet;

import java.util.Hashtable;
import com.sun.javadoc.*;
import java.io.IOException;
import java.io.Writer;

public class PackageWriter extends HtmlWriter {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/doclet/PackageWriter.java,v $ $revision$ $Date: 1998/11/02 15:54:49 $ Copyright 1998 INRIA.";

    public PackageWriter(PackageDoc aPackage,
			 String anOutputDirectory,
			 String aTemplateDirectory,
			 Hashtable aKnownTypeList,
			 ClassDoc[] anInterfaceList,
			 int anInterfaceSize,
			 ClassDoc[] aClassList,
			 int aClassSize,
			 ClassDoc[] anExceptionList,
			 int anExceptionSize,
			 ClassDoc[] anErrorList,
			 int anErrorSize) {
	super(anOutputDirectory,aTemplateDirectory,"package.html",aKnownTypeList);
	_fileName = "Package-" + aPackage.name() + ".html";
	_package = aPackage;
	_interfaces = anInterfaceList;
	_interfaceSize = anInterfaceSize;
	_classes = aClassList;
	_classSize = aClassSize;
	_exceptions = anExceptionList;
	_exceptionSize = anExceptionSize;
	_errors = anErrorList;
	_errorSize = anErrorSize;
    }

    protected void document() throws IOException {
	final Writer theStream = getOutputStream();
	copyTemplateHeader(theStream);
	title("Package " + _package.name() ,theStream);
	theStream.write("<TR><TD></TD><TD>" + 
			_package.commentText() + "</TD></TR>");
	summary(theStream,"Interface",_interfaces,_interfaceSize);
	summary(theStream,"Class",_classes,_classSize);
	summary(theStream,"Exception",_exceptions,_exceptionSize);
	summary(theStream,"Error",_errors,_errorSize);
	theStream.write("</TD></TR>\n");
	copyTemplateFooter(theStream);
    }

    private void summary(Writer aStream, String aType, ClassDoc[] aList, int aSize) throws IOException {
	if ( aSize > 0 ) {
	    section(aStream,aType + " Summary",null);
	    aStream.write("<TABLE VALIGN=\"TOP\">\n");
	    Utility.quickSort(aList,0,aSize-1);
	    for ( int theRank = 0; theRank < aSize; theRank++ ) {
		aStream.write("<TR><TD><UL><LI>");
		linkToStructure(aStream,aList[theRank]);
		aStream.write("</UL></TD><TD>" + 
			      shortComment(aList[theRank]) +
			      "</TD></TR>\n");
	    }
	    aStream.write("</TABLE>\n");
	    endSection(aStream);
	}
    }
    
    public String getOutputFileName() {
	return _fileName;
    }

    private String _fileName;
    private PackageDoc _package;
    private ClassDoc[] _interfaces;
    private int _interfaceSize;
    private ClassDoc[] _classes;
    private int _classSize;
    private ClassDoc[] _exceptions;
    private int _exceptionSize;
    private ClassDoc[] _errors;
    private int _errorSize;
    
}
