//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.event;

import java.util.EventListener;
import java.awt.event.KeyEvent;

/**
 * [PENDING: update the comments] EventMulticaster.
 * @level advanced
 */

public final class EventMulticaster implements PrintListenerInterface, HyperlinkListenerInterface, PointingListenerInterface, PointingMotionListenerInterface, KeyListenerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/event/EventMulticaster.java,v $ $revision$ $Date: 1998/11/02 15:54:55 $ Copyright 1998 INRIA.";
  
    public EventMulticaster(EventListenerInterface aListener,
			     EventListenerInterface anotherListener) {
	_listener = aListener;
	_otherListener = anotherListener;
    }

    public final static KeyListenerInterface addKeyListener(KeyListenerInterface aListener,
							    KeyListenerInterface anotherListener) {
	return (KeyListenerInterface)(add(aListener,anotherListener));
    }

    public final static KeyListenerInterface removeKeyListener(KeyListenerInterface aListener,
							       KeyListenerInterface anotherListener) {
	return (KeyListenerInterface)(remove(aListener,anotherListener));
    }

    public final static PointingListenerInterface addPointingListener(PointingListenerInterface aListener,
								      PointingListenerInterface anotherListener) {
	return (PointingListenerInterface)(add(aListener,anotherListener));
    }

    public final static PointingListenerInterface removePointingListener(PointingListenerInterface aListener,
									 PointingListenerInterface anotherListener) {
	return (PointingListenerInterface)(remove(aListener,anotherListener));
    }

    public final static PointingMotionListenerInterface addPointingMotionListener(PointingMotionListenerInterface aListener,
										  PointingMotionListenerInterface anotherListener) {
	return (PointingMotionListenerInterface)(add(aListener,anotherListener));
    }

    public final static PointingMotionListenerInterface removePointingMotionListener(PointingMotionListenerInterface aListener,
										     PointingMotionListenerInterface anotherListener) {
	return (PointingMotionListenerInterface)(remove(aListener,anotherListener));
    }
  
    public final static PrintListenerInterface addPrintListener(PrintListenerInterface aListener,
								PrintListenerInterface anotherListener) {
	return (PrintListenerInterface)(add(aListener,anotherListener));
    }
  
    public final static PrintListenerInterface removePrintListener(PrintListenerInterface aListener,
								   PrintListenerInterface anotherListener) {
	return (PrintListenerInterface)(remove(aListener,anotherListener));
    }

    public final static HyperlinkListenerInterface addHyperlinkListener(HyperlinkListenerInterface aListener,
									HyperlinkListenerInterface anotherListener) {
	return (HyperlinkListenerInterface)(add(aListener,anotherListener));
    }

    public final static HyperlinkListenerInterface removeHyperlinkListener(HyperlinkListenerInterface aListener,
									   HyperlinkListenerInterface anotherListener) {
	return (HyperlinkListenerInterface)(remove(aListener,anotherListener));
    }
  
    private final static EventListenerInterface add(EventListenerInterface aListener,
						    EventListenerInterface anotherListener) {
	if ( aListener == null ) {
	    return anotherListener;
	}
	else {
	    if ( anotherListener == null ) {
		return aListener;
	    }
	    else {
		return new EventMulticaster(aListener,anotherListener);
	    }
	}
    }
  
    private final static EventListenerInterface remove(EventListenerInterface aListener,
						       EventListenerInterface anotherListener) {
	if ( ( aListener == anotherListener ) || ( aListener == null ) ) {
	    return null;
	}
	else {
	    if ( aListener instanceof EventMulticaster ) {
		final EventMulticaster theMulticastListener = (EventMulticaster)aListener;
		if ( anotherListener == theMulticastListener._listener ) {
		    return theMulticastListener._otherListener;
		}
		else {
		    if ( anotherListener == theMulticastListener._otherListener ) {
			return theMulticastListener._listener;
		    }
		    else {
			final EventListenerInterface theListener = remove(theMulticastListener._listener,
									  anotherListener);
			final EventListenerInterface theOtherListener = remove(theMulticastListener._otherListener,
									       anotherListener);
			if ( ( theListener == theMulticastListener._listener ) &&
			     ( theOtherListener == theMulticastListener._otherListener ) ) {
			    return theMulticastListener;
			}
			else {
			    return add(theListener,theOtherListener);
			}
		    }
		}
	    }
	    else {
		return aListener;
	    }
	}
    }

    public final void keyTyped(KeyEvent anEvent) {
	if ( _listener != null ) {
	    ((KeyListenerInterface)_listener).keyTyped(anEvent);
	}
	if ( _otherListener != null ) {
	    ((KeyListenerInterface)_otherListener).keyTyped(anEvent);
	}
    }

    public final void keyPressed(KeyEvent anEvent) {
	if ( _listener != null ) {
	    ((KeyListenerInterface)_listener).keyPressed(anEvent);
	}
	if ( _otherListener != null ) {
	    ((KeyListenerInterface)_otherListener).keyPressed(anEvent);
	}
    }

    public final void keyReleased(KeyEvent anEvent) {
	if ( _listener != null ) {
	    ((KeyListenerInterface)_listener).keyReleased(anEvent);
	}
	if ( _otherListener != null ) {
	    ((KeyListenerInterface)_otherListener).keyReleased(anEvent);
	}
    }

    public final void mouseEntered(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingListenerInterface)_listener).mouseEntered(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingListenerInterface)_otherListener).mouseEntered(anEvent);
	}
    }
    public final void mouseExited(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingListenerInterface)_listener).mouseExited(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingListenerInterface)_otherListener).mouseExited(anEvent);
	}
    }
    public final void mousePressed(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingListenerInterface)_listener).mousePressed(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingListenerInterface)_otherListener).mousePressed(anEvent);
	}
    }
    public final void mouseReleased(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingListenerInterface)_listener).mouseReleased(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingListenerInterface)_otherListener).mouseReleased(anEvent);
	}
    }
    public final void mouseClicked(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingListenerInterface)_listener).mouseClicked(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingListenerInterface)_otherListener).mouseClicked(anEvent);
	}
    }

    public final void moving(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingMotionListenerInterface)_listener).moving(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingMotionListenerInterface)_otherListener).moving(anEvent);
	}
    }
    public final void startDragging(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingMotionListenerInterface)_listener).startDragging(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingMotionListenerInterface)_otherListener).startDragging(anEvent);
	}
    }
    public final void dragging(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingMotionListenerInterface)_listener).dragging(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingMotionListenerInterface)_otherListener).dragging(anEvent);
	}
    }
    public final void stopDragging(PointingEvent anEvent) {
	if ( _listener != null ) {
	    ((PointingMotionListenerInterface)_listener).stopDragging(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PointingMotionListenerInterface)_otherListener).stopDragging(anEvent);
	}
    }

    public final void linkEntered(HyperlinkEvent anEvent) {
	if ( _listener != null ) {
	    ((HyperlinkListenerInterface)_listener).linkEntered(anEvent);
	}
	if ( _otherListener != null ) {
	    ((HyperlinkListenerInterface)_otherListener).linkEntered(anEvent);
	}
    }
    public final void linkClicked(HyperlinkEvent anEvent) {
	if ( _listener != null ) {
	    ((HyperlinkListenerInterface)_listener).linkClicked(anEvent);
	}
	if ( _otherListener != null ) {
	    ((HyperlinkListenerInterface)_otherListener).linkClicked(anEvent);
	}
    }
    public final void linkExited(HyperlinkEvent anEvent) {
	if ( _listener != null ) {
	    ((HyperlinkListenerInterface)_listener).linkExited(anEvent);
	}
	if ( _otherListener != null ) {
	    ((HyperlinkListenerInterface)_otherListener).linkExited(anEvent);
	}
    }
  
    public final void printInitiate(PrintEvent anEvent) {
	if ( _listener != null ) {
	    ((PrintListenerInterface)_listener).printInitiate(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PrintListenerInterface)_otherListener).printInitiate(anEvent);
	}
    }
    public final void printAdvance(PrintEvent anEvent) {
	if ( _listener != null ) {
	    ((PrintListenerInterface)_listener).printAdvance(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PrintListenerInterface)_otherListener).printAdvance(anEvent);
	}
    }
    public final void printDone(PrintEvent anEvent) {
	if ( _listener != null ) {
	    ((PrintListenerInterface)_listener).printDone(anEvent);
	}
	if ( _otherListener != null ) {
	    ((PrintListenerInterface)_otherListener).printDone(anEvent);
	}
    }

    private EventListenerInterface _listener;
    private EventListenerInterface _otherListener;
  
}
