//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

import figue.box.GlyphInterface;
import figue.geometry.Dimension;
import figue.error.ShouldNotBeCalledError;

public abstract class StrategyComputeSizeTill {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/StrategyComputeSizeTill.java,v $ $revision$ $Date: 1998/11/02 15:55:06 $ Copyright 1998 INRIA.";
  
    protected StrategyComputeSizeTill() {
	;
    }

    public abstract void setRank(int aRank);
  
    public abstract boolean doAlgorithm(GlyphInterface aGlyph, int aRank);

    public abstract Dimension getExactOrEstimatedSize(GlyphInterface aGlyph, int aRank);

    public abstract void setFormatStatus(GlyphInterface aGlyph, int aRank);
  
    public final static StrategyComputeSizeTill THE_END = new StrategyComputeSizeTillTheEnd();
  
    public final static StrategyComputeSizeTill THE_RANK = new StrategyComputeSizeTillTheRank();
  
}

final class StrategyComputeSizeTillTheEnd extends StrategyComputeSizeTill {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/StrategyComputeSizeTill.java,v $ $revision$ $Date: 1998/11/02 15:55:06 $ Copyright 1998 INRIA.";

    public StrategyComputeSizeTillTheEnd() {
    }

    public final void setRank(int aRank) {
	throw new ShouldNotBeCalledError();
    }
  
    public final boolean doAlgorithm(GlyphInterface aGlyph, int aRank) {
	return ( aRank < aGlyph.numberOfChildren() );
    }

    public final Dimension getExactOrEstimatedSize(GlyphInterface aGlyph, int aRank) {
	return aGlyph.getSizeValue();
    }

    public final void setFormatStatus(GlyphInterface aGlyph, int aRank) {
	aGlyph.setClean();
    }
  
}

final class StrategyComputeSizeTillTheRank extends StrategyComputeSizeTill {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/StrategyComputeSizeTill.java,v $ $revision$ $Date: 1998/11/02 15:55:06 $ Copyright 1998 INRIA.";

    public StrategyComputeSizeTillTheRank() {
	_rank = -1;
    }

    public final boolean doAlgorithm(GlyphInterface aGlyph, int aRank) {
	return ( aRank <= _rank );
    }

    public final void setRank(int aRank) {
	_rank = aRank;
    }


    public final Dimension getExactOrEstimatedSize(GlyphInterface aGlyph, int aRank) {
	if ( aRank == _rank ) {
	    return aGlyph.getSizeEstimation();
	}
	else {
	    return aGlyph.getSizeValue();
	}
    }

    public final void setFormatStatus(GlyphInterface aGlyph, int aRank) {
	if ( aRank == _rank ) {
	    ;
	}
	else {
	    aGlyph.setClean();
	}
    }

    private int _rank;
  
}

