//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.resource.FontMetricInterface;
import figue.resource.FontInterface;
import figue.FacadeInterface;

import java.util.Hashtable;

public abstract class AwtPsContext extends Context {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/AwtPsContext.java,v $ $revision$ $Date: 1998/11/02 15:55:12 $ Copyright 1998 INRIA.";

    protected AwtPsContext(FacadeInterface aFacade,
			   String aFontName,
			   String aColorName,
			   String aBackgroundName) {
	super(aFacade,aFontName,aColorName,aBackgroundName);
	_registryFontMetrics = new Hashtable(Globals.AVERAGE_FONT_NUMBER,
					     Globals.FAST_ACCESS);
    }

    public final FontMetricInterface getFontMetric(FontInterface aFont) {
	final String theKey = aFont.getKey();
	FontMetricInterface theFontMetric = (FontMetricInterface)(_registryFontMetrics.get(theKey));
	if ( theFontMetric != null ) {
	    return theFontMetric;
	}
	theFontMetric = new AWTFontMetric(getAwtFontMetrics(aFont.getAwtFont()),aFont);
	_registryFontMetrics.put(theKey,theFontMetric);
	return theFontMetric;
    }

    protected abstract java.awt.FontMetrics getAwtFontMetrics(java.awt.Font aFont);
  
    public void dispose() {
	_registryFontMetrics.clear();
	_registryFontMetrics = null;
	super.dispose();
    }

    private Hashtable _registryFontMetrics;

}

class AWTFontMetric implements FontMetricInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/AwtPsContext.java,v $ $revision$ $Date: 1998/11/02 15:55:12 $ Copyright 1998 INRIA.";
  
    public AWTFontMetric(java.awt.FontMetrics aFontMetric,
			 FontInterface aFont) {
	_fontMetric = aFontMetric; 
	_height = _fontMetric.getAscent() + _fontMetric.getDescent();
	_font = aFont;
    }
  
    public final int getAscent() {
	return _fontMetric.getAscent();
    }
  
    public final int getDescent() {
	return _fontMetric.getDescent();
    }
  
    public final int bytesWidth(byte[] aByteArray) {
	return _fontMetric.bytesWidth(aByteArray,0,aByteArray.length);
    }

    public final int stringWidth(String aString) {
	return _fontMetric.stringWidth(aString);
    }
  
    public final int getHeight() {
	return _height;
    }

    public final FontInterface getFont() {
	return _font;
    }

    public final boolean equals(Object anObject) {
	if ( this == anObject ) {
	    return true;
	}
	if ( getClass() != anObject.getClass() ) {
	    return false;
	}
	return _font.equals(((AWTFontMetric)anObject).getFont());
    }
  
    private java.awt.FontMetrics _fontMetric;
    private int _height;
    private FontInterface _font;
  
}
