//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.resource.FontOwnerInterface;
import figue.resource.InitialFontOwnerInterface;
import figue.resource.ColorOwnerInterface;
import figue.resource.BackgroundOwnerInterface;
import figue.resource.FontMetricInterface;
import figue.resource.FontInterface;
import figue.resource.ColorInterface;
import figue.resource.PortableFontInterface;
import figue.selection.SelectionInterface;
import figue.geometry.PositionToFather;
import figue.geometry.PositionToBrother;
import figue.geometry.Position;
import figue.geometry.Point;
import figue.geometry.Rectangle;
import figue.format.DistanceInterface;
import figue.format.LazyDistance;
import figue.format.DecalageInterface;
import figue.format.LazyDecalage;
import figue.format.LazyDecalageToBrother;
import figue.format.Modification;
import figue.format.ModificationStatus;
import figue.error.ErrorHandlerInterface;
import figue.box.GlyphInterface;
import figue.box.LightweightInterface;
import figue.box.LightweightComponentAdapter;
import figue.FacadeInterface;

import java.util.Hashtable;
import java.awt.Graphics;

public interface ContextInterface extends FontOwnerInterface, InitialFontOwnerInterface, ColorOwnerInterface, BackgroundOwnerInterface, ErrorHandlerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/ContextInterface.java,v $ $revision$ $Date: 1998/11/02 15:55:13 $ Copyright 1998 INRIA.";

    public FacadeInterface getFacade();
  
    public void setCursor(Cursor aCursor);
    public FontInterface getFont();
    public void setFont(FontInterface aFont);
    public ColorInterface getColor();
    public void setColor(ColorInterface aColor);
    public ColorInterface getBackground();
    public void setBackground(ColorInterface aColor);
    public FontMetricInterface getFontMetric();
    public FontMetricInterface getFontMetric(FontInterface aFont);

    public ContextMementoInterface set(FontInterface aFont,
				       ColorInterface aColor, 
				       ColorInterface aBackground);
    public ContextMementoInterface apply(FontInterface aFont);
    public void applyFontToLightweightComponent(FontInterface aFont,
						LightweightInterface aLightweight);
    public ContextMementoInterface set(SelectionInterface aSelection);  
    public void set(SelectionInterface aSelection,
		    ContextMementoInterface aMemento);  
    public void setState(ContextMementoInterface aMemento);
    public void unapplyState(ContextMementoInterface aMemento);
    public ContextMementoInterface saveState();

    public void drawChars(char[] aCharArray, int x, int y);
    public void drawChar(char aChar, int x, int y);
    public void drawBytes(byte[] aByteArray, int x, int y);
    public void drawString(String aString, int x, int y);
    public void drawSymbol(PortableFontInterface aFont, String aString, int x, int y);
    public void fillRectangle(int x, int y, int width, int height);
    public void drawRectangle(int x, int y, int width, int height, ColorInterface aColor);
    public void drawLine(int x1, int y1, int x2, int y2);
    public void drawLightweightComponent(LightweightInterface aLightweight, int x, int y);
    public boolean isComplientWith(LightweightComponentAdapter aLightweight);
    
    public void flush();
    public void setRefreshZone(int x, int y, int width,int height);
    public Rectangle getRefreshZone(); 
    public Rectangle getClipBounds();

    public void touch();

    public void awaitMajorFormatAdvance(); 
    public void awaitClean(GlyphInterface aGlyph);
    public void awaitStable(GlyphInterface aGlyph);
    public void formatAdvance();
    public void majorFormatAdvance();
    public void lastMajorFormatAdvance();   /// +++
    public void lastMajorFormatInteractiveAdvance();   /// +++ 
    public void buildAdvance();
    public void buildComplete();

    public Hashtable[] getFlyWeightRegistryForChar();
    public Hashtable getFlyWeigthRegistryForString();
    public Hashtable getFlyWeigthRegistryForKeyWord();

    public void disposeMemento(ContextMementoInterface aMemento);
    public Point newPoint();
    public void disposePoint(Point aPoint);
    public PositionToFather newPositionToFather();
    public void disposePositionToFather(PositionToFather aPosition);
    public PositionToBrother newPositionToBrother(Position aPosition);
    public void disposePositionToBrother(PositionToBrother aPosition);
    public void disposeLazyDecalage(LazyDecalage aDecalage);
    public LazyDecalage newLazyDecalage(GlyphInterface aFather);
    public LazyDecalage newLazyDecalage(GlyphInterface aFather,
					DecalageInterface aDecalage);
    public void disposeLazyDecalageToBrother(LazyDecalageToBrother aDecalage);
    public LazyDecalageToBrother newLazyDecalageToBrother(GlyphInterface aFather,
							  GlyphInterface aBrother,
							  DecalageInterface aDecalage);
    public void disposeLazyDistance(LazyDistance aDistance);
    public LazyDistance newLazyDistance(GlyphInterface aGlyph,
					DistanceInterface aDistance,
					DecalageInterface aDecalage);
    public void disposePairGlyphPath(PairGlyphPath aPair);
    public PairGlyphPath newPairGlyphPath(GlyphInterface aGlyph);
    public void disposeModification(Modification aModification);
    public Modification newModification(ModificationStatus aStatus);
    public Modification newModification();

    public void init();
    public void dispose();

    public void setAwtContext(Graphics aGraphics);

}

