//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.resource.FontInterface;
import figue.resource.ColorInterface;
import figue.resource.PortableFontInterface;
import figue.geometry.Rectangle;
import figue.box.LightweightInterface;
import figue.InteractiveFacade;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Component;

public final class InteractiveContext extends AwtPsContext {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/InteractiveContext.java,v $ $revision$ $Date: 1998/11/02 15:55:15 $ Copyright 1998 INRIA.";

    public  InteractiveContext(InteractiveFacade aFacade,
			       String aFontName,
			       String aColorName,
			       String aBackgroundName,
			       InteractiveClientInterface aViewer) {
	super(aFacade,aFontName,aColorName,aBackgroundName);
	_component = aViewer;
	_graphics = null;
	_offImage = null;
	_offGraphics = null;
	_offImageBounds = new Rectangle();
	_singleChar = new char[1];
	_graphicsForFontMetrics = null;
	_cacheForClip = new Rectangle();
    }
    
    public final void init() {
	while ( true ) {
	    _graphicsForFontMetrics = _component.getGraphics();
	    if ( _graphicsForFontMetrics != null ) {
		break;
	    }
	    else {
		try {
		    System.out.print(".");
		    System.out.flush();
		    java.lang.Thread.sleep(250);
		}
		catch ( InterruptedException anException ) {
		    ;
		}
	    }
	}
	getFacade().initDone();
    }
    
    public final void setCursor(Cursor aCursor) {
	_component.setCursor(aCursor);
    }
    
    public final void setFont(FontInterface aFont) {
	super.setFont(aFont);
	_offGraphics.setFont(aFont.getAwtFont());
    }
    
    public final void setColor(ColorInterface aColor) {
	super.setColor(aColor);
	_offGraphics.setColor(aColor.getAwtColor()); 
    }
    
    protected final java.awt.FontMetrics getAwtFontMetrics(java.awt.Font aFont) {
	return _graphicsForFontMetrics.getFontMetrics(aFont);
    }
    
    public final void drawBytes(byte[] aByteArray, int x, int y) {
	_offGraphics.drawBytes(aByteArray,0,aByteArray.length,x,y);
    }
    
    public final void drawString(String aString, int x, int y) {
	_offGraphics.drawString(aString,x,y);
    }

    public final void drawSymbol(PortableFontInterface aFont, String aString, int x, int y) {
	aFont.draw(_offGraphics,aString,x,y);
    }
    
    public final void drawChars(char[] aCharArray, int x, int y) {
	_offGraphics.drawChars(aCharArray,0,aCharArray.length,x,y);
    }
    
    public final void drawChar(char aChar, int x, int y) {
	_singleChar[0] = aChar;
	_offGraphics.drawChars(_singleChar,0,1,x,y);
    }
    
    public final void fillRectangle(int x, int y, int width, int height) {
	_offGraphics.setColor(getBackground().getAwtColor()); 
	_offGraphics.fillRect(x,y,width,height);
	_offGraphics.setColor(getColor().getAwtColor());
    }
    
    public final void drawRectangle(int x, int y, int width, int height, ColorInterface aColor) {
	_offGraphics.setColor(aColor.getAwtColor());
	_offGraphics.drawRect(x,y,width,height);
	_offGraphics.setColor(getColor().getAwtColor());
    }
    
    public final void drawLine(int x1, int y1, int x2, int y2) {
	_offGraphics.drawLine(x1,y1,x2,y2);
    }
    
    public final void drawLightweightComponent(LightweightInterface aLightweight, int x, int y) {
	aLightweight.setForeground(getColor().getAwtColor());
	aLightweight.setBackground(getBackground().getAwtColor());
	aLightweight.setFont(getFont().getAwtFont());
	aLightweight.setLocation(0,0);
	_offGraphics.translate(x,y);
	aLightweight.update(_offGraphics);
	_offGraphics.translate(-x,-y);
    }
    
    public final void applyFontToLightweightComponent(FontInterface aFont,
						      LightweightInterface aLightweight) {
	aLightweight.setFont(aFont.getAwtFont());
    }

    public final void setAwtContext(Graphics aGraphics) {
	_graphics = aGraphics;
    }
    
    public final Rectangle getRefreshZone() {
	return _offImageBounds;
    }
    
    public final Rectangle getClipBounds() {
	final java.awt.Rectangle theClip = _graphics.getClipBounds();
	_cacheForClip.setBounds(theClip.x,theClip.y,theClip.width,theClip.height);
	return _cacheForClip;
    }
    
    public final void setClipBounds(int x, int y, int width,int height) {
	_graphics.setClip(x,y,width,height);
    }
    
    public final void setRefreshZone(int x, int y, int width,int height) {
	_offImageBounds.setBounds(x,y,width,height);
	if ( ( _offImage == null ) || 
	     ( _offImage.getHeight(null) < height ) || 
	     ( _offImage.getWidth(null) < width ) ) {
	    _offImage = _component.createImage(width,height);
	}
	else {
	    _offImage.flush();
	}
	if ( _offGraphics != null ) {
	    _offGraphics.dispose();
	}
	_offGraphics = _offImage.getGraphics();
	_offGraphics.setColor(getBackground().getAwtColor());
	_offGraphics.fillRect(0,0,width,height);
	_offGraphics.setFont(getFont().getAwtFont());
	_offGraphics.setColor(getColor().getAwtColor());
	_offGraphics.translate(-x,-y);
	_offGraphics.setClip(x,y,width,height);
    }
    
    public final void flush() {
	_graphics.drawImage(_offImage,_offImageBounds.getX(),_offImageBounds.getY(),null);
    }
    
    public final void touch() {
	_component.touch();
    }
    
    public final void majorFormatAdvance() { 
	_component.updateScrollbars();
	_component.refresh();
	getFacade().suspendFormatting();
	getFacade().waitUntilFormattingResumed();
	super.majorFormatAdvance();
    }
    
    public final void lastMajorFormatAdvance() { /// +++
	_component.updateScrollbars();
	_component.refresh();
	super.lastMajorFormatAdvance();
    }
    
    public final void lastMajorFormatInteractiveAdvance() { /// +++
	_component.updateScrollbars();
	super.lastMajorFormatInteractiveAdvance();
    }
    
    
    public final void dispose() {
	_component.getParent().remove((Component)_component); /// BEURK
	_component.dispose();
	_component = null;
	if ( _graphics != null ) {
	    _graphics.dispose();
	}
	_graphics = null;
	if ( _offImage != null ) {
	    _offImage.flush();
	}
	_offImage = null;
	if ( _offGraphics != null ) {
	    _offGraphics.dispose();
	}
	_offGraphics = null;
	_offImageBounds = null;
	_singleChar = null;
	if ( _graphicsForFontMetrics != null ) {
	    _graphicsForFontMetrics.dispose();
	    _graphicsForFontMetrics = null;
	}
	_cacheForClip = null;
	super.dispose();
    }
    
    private InteractiveClientInterface _component; 
    private Graphics _graphics;
    private Image _offImage;
    private Graphics _offGraphics;
    private Rectangle _offImageBounds;
    private char[] _singleChar;
    private Graphics _graphicsForFontMetrics;
    private Rectangle _cacheForClip;
    
}
