//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.box.GlyphInterface;
import figue.path.SelectionPathInterface;
import figue.path.SelectionPath;
import figue.path.PathInterface;
import figue.path.TouchSelection;
import figue.path.ShrinkSelection;
import figue.path.ExtendSelection;
import figue.path.InverseSelection;
import figue.error.ShouldNotBeCalledError;
import figue.FacadeInterface;

public final class PathSelectionVisitor extends PathVisitorIncremental {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/PathSelectionVisitor.java,v $ $revision$ $Date: 1998/11/02 15:55:16 $ Copyright 1998 INRIA.";
    
    public PathSelectionVisitor(FacadeInterface aFacade) {
	super(aFacade);
    }
    
    public final void extendSelection(ExtendSelection anExtendOperation,
				      GlyphInterface aGlyph,
				      SelectionPathInterface aSelectionPath) {
	updateRefreshZone(aGlyph);
	aSelectionPath.addSelection(anExtendOperation.getSelection());
    }
    public final void shrinkSelection(ShrinkSelection aShrinkOperation,
				      GlyphInterface aGlyph,
				      SelectionPathInterface aSelectionPath) {
	updateRefreshZone(aGlyph);
	aSelectionPath.removeSelection(aShrinkOperation.getSelection(),
				       this);
    }
    
    public void inverseSelection(InverseSelection anInverseOperation,
				 GlyphInterface aGlyph,
				 SelectionPathInterface aSelectionPath) {
	updateRefreshZone(aGlyph);
	aSelectionPath.inverseSelection(anInverseOperation.getSelection(),
					this);
    }

    public final void touchSelection(TouchSelection aTouchOperation,
				     GlyphInterface aGlyph,
				     SelectionPathInterface aSelectionPath) {
	updateRefreshZone(aGlyph);
    }
    
    public final void down(GlyphInterface aGlyph,
			   SelectionPathInterface aSelectionPath,
			   PathInterface aPath) {
	final int thePathRank = aPath.getRank();
	SelectionPathInterface theNextSelectionPath = aSelectionPath.search(thePathRank);
	if ( theNextSelectionPath == null ) {
	    theNextSelectionPath = new SelectionPath(thePathRank);
	    aSelectionPath.addChild(theNextSelectionPath);
	}
	doTranslateZones(aGlyph,thePathRank);
	aPath.accept(this,
		     aGlyph.getChild(thePathRank),
		     theNextSelectionPath);
	undoTranslateZones(aGlyph,thePathRank);
    }
    
    public final void down(GlyphInterface aGlyph,
			   PathInterface aPath) {
	reportError(new ShouldNotBeCalledError());
    }
    
}
