//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.path.SelectionPathInterface;
import figue.path.InsertGlyph;
import figue.path.ReplaceGlyph;
import figue.path.RemoveGlyph;
import figue.path.MarkGlyph;
import figue.path.TouchGlyph;
import figue.path.ExtendSelection;
import figue.path.ShrinkSelection;
import figue.path.InverseSelection;
import figue.path.TouchSelection;
import figue.path.MarkStable;
import figue.path.MarkUnstable;
import figue.path.NotSupportedOperationError;
import figue.error.ErrorHandler;
import figue.error.ErrorHandlerInterface;
import figue.box.GlyphInterface;
import figue.FacadeInterface;

public abstract class PathVisitor extends ErrorHandler implements PathVisitorInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/PathVisitor.java,v $ $revision$ $Date: 1998/11/02 15:55:17 $ Copyright 1998 INRIA.";

    protected PathVisitor(FacadeInterface aFacade) {
	_facade = aFacade;
	_emptyLeaves = new StackAsync(5,5);
	_context = null;
    }

    public final void init() {
	_context = getFacade().getContext();
    }
  
    public final FacadeInterface getFacade() {
	return _facade;
    }

    public final ContextInterface getContext() {
	return _context;
    }

    public final void addEmptyLeaf(SelectionPathInterface aLeaf) {
	_emptyLeaves.push(aLeaf);
    }

    public final StackInterface getEmptyLeaves() {
	return _emptyLeaves;
    }

    public void extendSelection(ExtendSelection anExtendOperation,
				GlyphInterface aGlyph,
				SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(anExtendOperation));
    }

    public void extendSelection(ExtendSelection anExtendOperation,
				GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(anExtendOperation));
    }
  
    public void shrinkSelection(ShrinkSelection aShrinkOperation,
				GlyphInterface aGlyph,
				SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(aShrinkOperation));
    }

    public void shrinkSelection(ShrinkSelection aShrinkOperation,
				GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(aShrinkOperation));
    }

    public void inverseSelection(InverseSelection anInverseOperation,
				 GlyphInterface aGlyph,
				 SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(anInverseOperation));
    }

    public void inverseSelection(InverseSelection anInverseOperation,
				 GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(anInverseOperation));
    }

    public void touchSelection(TouchSelection aTouchOperation,
			       GlyphInterface aGlyph,
			       SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(aTouchOperation));
    }
    public void touchSelection(TouchSelection aTouchOperation,
			       GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(aTouchOperation));
    }
  
    public void insertGlyph(InsertGlyph anInsertOperation,
			    GlyphInterface aGlyph,
			    SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(anInsertOperation));
    }

    public void insertGlyph(InsertGlyph anInsertOperation,
			    GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(anInsertOperation));
    }
  
    public void replaceGlyph(ReplaceGlyph aReplaceOperation,
			     GlyphInterface aGlyph,
			     SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(aReplaceOperation));
    }

    public void replaceGlyph(ReplaceGlyph aReplaceOperation,
			     GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(aReplaceOperation));
    }

    public void touchGlyph(TouchGlyph aTouchOperation,
			   GlyphInterface aGlyph,
			   SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(aTouchOperation));
    }
  
    public void touchGlyph(TouchGlyph aTouchOperation,
			   GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(aTouchOperation));
    }
  
    public void removeGlyph(RemoveGlyph aRemoveOperation,
			    GlyphInterface aGlyph,
			    SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(aRemoveOperation));
    }

    public void removeGlyph(RemoveGlyph aRemoveOperation,
			    GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(aRemoveOperation));
    }

    public void markGlyph(MarkGlyph aMarkOperation,
			  GlyphInterface aGlyph,
			  SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(aMarkOperation));
    }

    public void markGlyph(MarkGlyph aMarkOperation,
			  GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(aMarkOperation));
    }
  
    public void markStable(MarkStable aMarkStableOperation,
			   GlyphInterface aGlyph,
			   SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(aMarkStableOperation));
    }

    public void markStable(MarkStable aMarkStableOperation,
			   GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(aMarkStableOperation));
    }
  
    public void markUnstable(MarkUnstable aMarkUnstableOperation,
			     GlyphInterface aGlyph,
			     SelectionPathInterface aSelectionPath) {
	reportError(new NotSupportedOperationError(aMarkUnstableOperation));
    }

    public void markUnstable(MarkUnstable aMarkUnstableOperation,
			     GlyphInterface aGlyph) {
	reportError(new NotSupportedOperationError(aMarkUnstableOperation));
    }

    public final ErrorHandlerInterface getNextErrorHandler() {
	return _facade;
    }

    public final boolean hasNextErrorHandler() {
	return ( _facade != null);
    }

    public final boolean canHandleError() {
	return false;
    }

    protected final void removeSelections(GlyphInterface aGlyph,
					  SelectionPathInterface aSelectionPath) {
	aSelectionPath.removeAllSelections(this);
    }

    public void dispose() {
	_facade = null;
	_emptyLeaves = null;
	_context = null;
    }

    private FacadeInterface _facade;
    private StackInterface _emptyLeaves;
    private ContextInterface _context;
  
}
