header {
//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.io.xml;

import figue.box.*;
import figue.path.*;
import figue.internal.*;
import figue.*;

import java.util.Enumeration;

}
options {
language="Java";
}
class FigueParser extends Parser;
options {
tokenVocabulary=figueTokens;
buildAST=true;
k=2;
}

document		: 	prolog!
				element
				( misc! )*
				EOF!
			;

prolog			:	XML_DECL!
				( misc! )*
			;

element			:	o:OPEN_TAG!
				( content )*
				c:CLOSE_TAG!
				{ o.getText().equals(c.getText()) }?
				{ ! ((TokenWithAttributes)o).hasRedefinedAttributes() }?
				{ final TokenWithAttributes theToken = (TokenWithAttributes)o;
				  final AST theNode = Utility.decodeFigueTypeFromXml(astFactory,theToken.getText());
				  final AST theAttributes = Utility.decodeFigueAttributesFromXml(astFactory,theToken);
				  if ( theAttributes != null ) {
					#element = #(theNode, theAttributes , #element);
				  }
				  else {
					#element = #(theNode, #element);
				  }
				}
			|	e:EMPTY_ELEMENT
				{ ! ((TokenWithAttributes)e).hasRedefinedAttributes() }?
				{ final TokenWithAttributes theToken = (TokenWithAttributes)e;
				  final AST theNode = Utility.decodeFigueTypeFromXml(astFactory,theToken.getText());
				  final AST theAttributes = Utility.decodeFigueAttributesFromXml(astFactory,theToken);
				  if ( theAttributes!= null ) {
				    	#element = #(theNode, theAttributes);
				  }
				  else {
				   	#element = theNode;
				  }
				}
			;

content			:	element
			|	CDATA_SECTION
			|	PROCESSING_INST
			|	reference
			|	CHAR_DATA
			|	WHITE_SPACE
			;

reference		:	REFERENCE
			;

misc!			:	PROCESSING_INST
			|	COMMENT
			|	WHITE_SPACE
			;

imaginaryTokenDef	:	ELEMENT
			|	ATTRIBUTE_LIST
			|	STRING
			;


class FigueWalker extends TreeParser;
options {
tokenVocabulary=figueTokens;
buildAST=true;
}
path
returns [ PathInterface r] 
{ r = new Path(); PathOperationInterface O; PathInterface P; }
		:	#( PATH
			   #( ATTRIBUTE_LIST I0:A_RANK )
			   { r.setRank(Utility.parseInt(I0.getText())); }
			   ( O=operation { r.addOperation(O); } )*
			   ( P=path { r.addChild(P); } )*
			)
		;

operation
returns [ PathOperationInterface r ]
{ r = null; GlyphInterface G; }
		:	#( INSERT_GLYPH 
				#( ATTRIBUTE_LIST I1:A_RANK ) 
				G=glyph_or_datas )
			{ r = new InsertGlyph(G,Utility.parseInt(I1.getText())); }
		|	#( REMOVE_GLYPH 
				#( ATTRIBUTE_LIST I2:A_RANK ) )
			{ r = new RemoveGlyph(Utility.parseInt(I2.getText())); }
		|	#( REPLACE_GLYPH 
				G=glyph_or_datas )
			{ r = new ReplaceGlyph(G); }
		;

facade [ FacadeInterface aFacade]
{ GlyphInterface r = null; }
		:	#( FACADE
			   ( resources[aFacade] )?
			   ( selections[aFacade] )?
			   ( r=root { aFacade.getRootGlyph().addChild(r); } )?
			)
		;

root
returns [ GlyphInterface r ]
{ r = null; }
		:	#( ROOT
			   r=glyph_or_datas 
			)
		;

glyph
returns [ GlyphInterface r ]
{ r = null; VectorGlyph g; GlyphInterface c; String s; String v1 = null;
  String u1 = null; String v2 = null; String u2 = null; String v3 = null;
  String u3 = null; String v4 = null; String u4 = null; }
		:	#( HORIZONTAL
			   #( ATTRIBUTE_LIST 
				( V1:A_INTERWORD { v1 = V1.getText(); }
				  ( U1:A_UNIT { u1 = U1.getText(); } )? )* )
			   g=glyphs_and_datas
			   { r = Utility.newHorizontal(g,v1,u1); }
			)
		|	#( VERTICAL
			   #( ATTRIBUTE_LIST 
				( ( V01:A_INDENT { v1 = V01.getText(); }
				    ( U01:A_UNIT { u2 = U01.getText(); } )? )
				  | ( V02:A_INTERLINE { v2 = V02.getText(); }
				      ( U02:A_UNIT { u2 = U02.getText(); } )? ) )* )
			   g=glyphs_and_datas
			   { r = Utility.newVertical(g,v1,u1,v2,u2); }
			)
		|	#( PARAGRAPH
			   #( ATTRIBUTE_LIST 
				( ( V11:A_INTERWORD { v1 = V11.getText(); }
				    ( U11:A_UNIT { u1 = U11.getText();} )? )
				| ( V12:A_INDENT { v2 = V12.getText(); } 
				    ( U12:A_UNIT { u2 = U12.getText(); } )? )
				| ( V13:A_INTERLINE { v3 = V13.getText(); }
				    ( U13:A_UNIT { u3 = U13.getText(); } )? ) )* )
			   g=glyphs_and_datas
			   { r = Utility.newParagraph(g,v1,u1,v2,u2,v3,u3); }
			)
		|	( #( CHANGE_CONTEXT
			   #( ATTRIBUTE_LIST A_STYLE )
			) )=>
			#( CHANGE_CONTEXT
			   #( ATTRIBUTE_LIST V21:A_STYLE )
			   c=glyph_or_datas
			   { r = Utility.newChangeContext(c,V21.getText()); }
			)
		|	#( CHANGE_CONTEXT
			   #( ATTRIBUTE_LIST 
				( V31:A_FONT { v1 = V31.getText(); } 
				| V32:A_FOREGROUND { v2 = V32.getText(); }
				| V33:A_BACKGROUND { v3 = V33.getText(); } )+ )
			   c=glyph_or_datas
			   { r = Utility.newChangeContext(c,v1,v2,v3); }
			)
		|	#( INSETS
			   #( ATTRIBUTE_LIST 
				( ( V41:A_SPACE_BEFORE { v1 = V41.getText(); }
				    ( U41:A_UNIT { u1 = U41.getText(); } )? )
				| ( V42:A_SPACE_AFTER { v2 = V42.getText(); }
				    ( U42:A_UNIT { u2 = U42.getText(); } )? )
				| ( V43:A_SPACE_TO_LEFT { v3 = V43.getText(); }
				    ( U43:A_UNIT { u3 = U43.getText(); } )? )
				| ( V44:A_SPACE_TO_RIGHT { v4 = V44.getText(); }
				    ( U44:A_UNIT { u4 = U44.getText(); } )? ) )* )
			   c=glyph_or_datas
			   { r = Utility.newInsets(c,v1,u1,v2,u2,v3,u3,v4,u4); }
			)
		|	#( KEYWORD
			   #( ATTRIBUTE_LIST 
				( V51:A_STYLE { v1 = V51.getText(); } 
				| V52:A_TEXT { v2 = V52.getText(); } )* )
			   { r = Utility.newKeyWord(v2,v1);}
			)
		|	#( HYPERLINK
			   #( ATTRIBUTE_LIST 
				( V61:A_ACTIVE_COLOR { v1 = V61.getText(); } 
				| V62:A_VISITED_COLOR { v2 = V62.getText(); }
				| V63:A_SELECTED_COLOR { v3 = V63.getText(); }
				| V64:A_FONT { v4 = V64.getText(); } )* )
			   c=glyph_or_datas
			   { r = Utility.newHyperlink(c,v1,v2,v3,v4); }
			)
		|	( #( GLYPH
			   #( ATTRIBUTE_LIST ( #( STRING STRING ) )* ) ) ) =>
			#( GLYPH
			   #( ATTRIBUTE_LIST ( #( STRING STRING ) )* )
			   g=glyphs_and_datas
			)
		|	#( GLYPH
			   g=glyphs_and_datas
			)
		;

glyphs_and_datas
returns [ VectorGlyph r ]
{
 r = null;
 VectorGlyph theGlyphs = new VectorGlyph();
 StringBuffer theCurrentAtom = new StringBuffer();
 String d;
 GlyphInterface g;
}
		:	( g=glyph { theGlyphs.addElement(g); }
			  | d=data { if (d != null ) {
					theCurrentAtom.append(d);
				     }
				     else {
					if ( theCurrentAtom.length() > 0 ) {
						theGlyphs.addElement(Atom.newAtom(theCurrentAtom.toString()));
						theCurrentAtom.setLength(0);
					}
				     }; } )*
			{ if ( theCurrentAtom.length() > 0 ) {
				theGlyphs.addElement(Atom.newAtom(theCurrentAtom.toString()));
				theCurrentAtom.setLength(0);
			  };
			  r = theGlyphs;
			}
		;

glyph_or_datas
returns [ GlyphInterface r ]
{ r = null; String d; StringBuffer theCurrentAtom = new StringBuffer(); }
		:	( r=glyph )? 
		|	( d=data { theCurrentAtom.append(d); } )+ 
			{ r = Atom.newAtom(theCurrentAtom.toString()); }
		;

data
returns [ String r ]
{ r = null; }
		:	x:CDATA_SECTION { r = x.getText(); }
		|	PROCESSING_INST { r = null; }
		|	y:REFERENCE { r = y.getText(); }
		|	w:CHAR_DATA { r = w.getText(); }
		|	WHITE_SPACE { r = null; }
		;

selections [ FacadeInterface aFacade]
		:	#( SELECTIONS
			   ( selection[aFacade] )*
			)
		;

selection [ FacadeInterface aFacade]
{ String theName = null; String thePriority = null; String theFlag = null; String theColor = null; String theBackground = null; }
		:	#( SELECTION
			   #( ATTRIBUTE_LIST 
				( ( n:A_NAME { theName = n.getText(); } ) 
				  | ( f:A_FOREGROUND  { theColor = f.getText(); } )
				  | ( b:A_BACKGROUND { theBackground = b.getText(); } )
				  | ( p:A_PRIORITY { thePriority = p.getText(); } )
				  | ( v:A_VISIBLE { theFlag = v.getText(); } ) )* )
			)
			{ Utility.newSelection(aFacade,theName,thePriority,theColor,theBackground,theFlag) }?
		;

resources [ FacadeInterface aFacade]
		:	#( RESOURCES
			   fonts_colors_styles[aFacade]
			)
		;

fonts_colors_styles [ FacadeInterface aFacade]
		:	( fonts[aFacade] )?
			( colors[aFacade] )?
			( styles[aFacade] )?
		;

fonts [ FacadeInterface aFacade]
		:	#( FONTS
			   ( font[aFacade] )*
			)
		;

font [ FacadeInterface aFacade]
{ String theName = null; String theSize = null; String theStyle = null; String theFamily = null; String theFlag = null; }
		:	#( FONT
			   #( ATTRIBUTE_LIST 
				( ( n:A_NAME {theName = n.getText(); } )
				  | ( p:A_SIZE { theSize = p.getText(); } )
				  | ( s:A_STYLE { theStyle = s.getText(); } )
				  | ( f:A_FAMILY { theFamily = f.getText(); } )
				  | ( b:A_UNDERLINED { theFlag = b.getText(); } ) )* )
			)
			{ Utility.newFont(aFacade,theName,theFamily,theStyle,theSize,theFlag) }?
		;

colors [ FacadeInterface aFacade]
		:	#( COLORS
			   ( color[aFacade] )*
			)
		;

color [ FacadeInterface aFacade]
{ String theName = null; String theRed = null; String theGreen = null; String theBlue = null; }
		:	#( COLOR
			   #( ATTRIBUTE_LIST  
				( ( n:A_NAME { theName = n.getText(); } )
				  | ( r:A_RED { theRed = r.getText(); } )
				  | ( g:A_GREEN { theGreen = g.getText(); } )
				  | ( b:A_BLUE { theBlue = b.getText(); } ) )* )
			)
			{ Utility.newColor(aFacade,theName,theRed,theGreen,theBlue) }?
		;

styles[ FacadeInterface aFacade]
		:	#( STYLES
			   ( style[aFacade] )*
			)
		;

style[ FacadeInterface aFacade]
{ String theName = null; String theFont = null; String theForeground = null; String theBackground = null; }
		:	#( STYLE
			   #( ATTRIBUTE_LIST 
				( ( n:A_NAME { theName = n.getText(); } )
				  | ( f:A_FONT { theFont = f.getText(); } )
				  | ( c:A_FOREGROUND { theForeground = c.getText(); } )
				  | ( b:A_BACKGROUND { theBackground = b.getText(); } ) )* )
			)
			{ Utility.newStyle(aFacade,theName,theFont,theForeground,theBackground) }?
		;

imaginaryTokenDef	:	FACADE
			|	SELECTIONS
			|	SELECTION
			|	RESOURCES
			|	FONTS
			|	FONT
			|	COLORS
			|	COLOR
			|	STYLES
			|	STYLE
			|	ROOT
			|	GLYPH
			|	VERTICAL
			|	HORIZONTAL
			|	PARAGRAPH
			|	CHANGE_CONTEXT
			|	KEYWORD
			|	INSETS
			|	HYPERLINK
			|	PATH
			|	INSERT_GLYPH
			|	REMOVE_GLYPH
			|	REPLACE_GLYPH
			|	TOUCH_GLYPH
			|	MARK_GLYPH
			|	MARK_STABLE
			|	MARK_UNSTABLE
			|	EXTEND_SELECTION
			|	INVERSE_SELECTION
			|	SHRINK_SELECTION
			|	A_NAME
			|	A_RED
			|	A_GREEN
			|	A_BLUE
			|	A_SIZE
			|	A_STYLE
			|	A_FAMILY
			|	A_UNDERLINED
			|	A_FONT
			|	A_FOREGROUND
			|	A_BACKGROUND
			|	A_UNIT
			|	A_INDENT
			|	A_INTERWORD
			|	A_INTERLINE
			|	A_VISIBLE
			|	A_PRIORITY
			|	A_SPACE_TO_LEFT
			|	A_SPACE_TO_RIGHT
			|	A_SPACE_BEFORE
			|	A_SPACE_AFTER
			|	A_TEXT
			|	A_ACTIVE_COLOR
			|	A_VISITED_COLOR
			|	A_SELECTED_COLOR
			|	A_RANK
			;

class FigueLexer extends Lexer;
options {
tokenVocabulary=figueTokens;	
charVocabulary='\3'..'\377';
caseSensitive=true;
k=3;
}

protected
XML_DECL!	:	PROLOG_START
			( WHITE_SPACE )?
			VERSION
			( ( WHITE_SPACE )? ENCODING )?
			( ( WHITE_SPACE )? SDD )?
			( WHITE_SPACE )?
			PROLOG_END
		;

protected
PROLOG_START!	:	"<?" XML
		;

protected
XML!		:	( 'X' | 'x' ) ( 'M' | 'm' ) ( 'L' | 'l' )
		;

protected
VERSION		:	"version"!
			( WHITE_SPACE! )?
			'='!
			( WHITE_SPACE! )?
			( ( '"'! ( NAME_CHARACTER )+ '"'! ) |
			  ( '\''! ( NAME_CHARACTER )+ '\''! ) )
		;

protected
ENCODING!	:	"encoding"!
			( WHITE_SPACE! )?
			'='!
			( WHITE_SPACE! )?
			( ( '"'! ( NAME_CHARACTER )* '"'! ) |
			  ( '\''! ( NAME_CHARACTER )* '\''! ) )
		;

protected
SDD!		:	"standalone"!
			( WHITE_SPACE! )?
			'='!
			( WHITE_SPACE! )?
			( ( '"'! YES_OR_NO '"'! ) |
			  ( '\''! YES_OR_NO '\''! ) )
		;

protected
YES_OR_NO!	:	( "yes" | "no" )
		;

protected
PROLOG_END!	:	"?>"
		;

protected
DOC_TYPE	:	"<!DOCTYPE"!
			WHITE_SPACE!
			NAME
		;

protected
OPEN_TAG
{
 Attributes a = null;
}
		:	t:START_TAG
			( a=ATTRIBUTES ) ? 
			( WHITE_SPACE! )?
			'>'!
			 { CommonToken theToken =
					new TokenWithAttributes(OPEN_TAG,
							   	t.getText(),
                                                            	a);
			  $setToken(theToken); }
		;

protected
EMPTY_TAG
{
 Attributes a = null;
}
		:	t:START_TAG
			( a=ATTRIBUTES ) ?
			( WHITE_SPACE! )? 
			"/>"!
			{ CommonToken theToken = 
					new TokenWithAttributes(EMPTY_TAG,
							    	t.getText(),
                                                            	a);
			  $setToken(theToken); }
		;

OPEN_TAG_OR_EMPTY_ELEMENT
		:	( START_TAG
			  ( ATTRIBUTES! ) ?
			  ( WHITE_SPACE )? 
			  '>' ) =>

			x:OPEN_TAG
			{ CommonToken theToken =
					new TokenWithAttributes(OPEN_TAG,
							   	x.getText(),
                                                            	((TokenWithAttributes)x).getAttributes());
			  $setToken(theToken); }
		|	y:EMPTY_TAG
			{ CommonToken theToken =
					new TokenWithAttributes(EMPTY_ELEMENT,
							   	y.getText(),
                                                            	((TokenWithAttributes)y).getAttributes());
			 $setToken(theToken); }
		;

protected
START_TAG	:	'<'! NAME
		;

CLOSE_TAG	:	"</"! NAME ( WHITE_SPACE! )? '>'!
		;

protected
ATTRIBUTES! returns [ Attributes r ]
{
 r = new Attributes();
}
		:	( ( WHITE_SPACE )? ATTRIBUTE[r] )+
		;

protected
ATTRIBUTE! [ Attributes a ]
		:	n:NAME
			'='
			v:ATTRIBUTE_VALUE
			{ a.put(n.getText(),v.getText()); }
		;

protected
ATTRIBUTE_VALUE	:	( ( '"'! ( ~( '<' | '&' | '"' ) | REFERENCE )* '"'!)
			  | ( '\''! ( ~( '<' | '&' | '\'' ) | REFERENCE )* '\''! ) )
		;

protected
NAME		:	( LETTER | '_' | ':' ) ( NAME_CHARACTER )* 
		;

protected
NAME_CHARACTER	:	( LETTER | DIGIT | '_' | '-' | '.' | ':' )
		;

protected
LETTER 		:	( UPPER_C_LETTER | LOWER_C_LETTER );

protected
UPPER_C_LETTER	:	( 'a'..'z' )
			;

protected
LOWER_C_LETTER 	:	( 'A'..'Z' )
			;

protected
DIGIT		:	( '0'..'9' )
		;

protected
INT		:	( DIGIT )+
		;

protected
ALPHA_NUM	:	( DIGIT | LETTER )+
		;

CHAR_DATA	:	( ~( '<' | '&' | ' ' | '\t' | '\n' | '\r' ) )+
		;

WHITE_SPACE 	:	( ' ' | '\t' | '\n' | '\r' )+
			{ $setText("`" + $getText + "'"); }
		;

REFERENCE!	:	x:ENTITY_REF { $setText(Utility.getEntity(x.getText())); }
		|	y:CHAR_REF { $setText(y.getText()); }
		;

protected
ENTITY_REF!	:	'&' x:NAME ';'
			{ $setText(x.getText()); }
		;

protected
CHAR_REF	:	"&#"! x:INT ';'!
			{ $setText(Utility.charReference(x.getText(),10)); }
		|	"&#x"! y:ALPHA_NUM ';'!
			{ $setText(Utility.charReference(y.getText(),16)); }
		;

protected
PARAM_ENTITY_REF:	'%'! NAME ';'!
		;

COMMENT		:	COMMENT_START! 
			( ~('-') | ( '-' (~('-')) ) )* 
			COMMENT_END!
		;

protected
COMMENT_START!	:	"<!--"
		;

protected
COMMENT_END!	:	"-->"
		;

CDATA_SECTION   :       CDATA_START! 
			( ~(']') 
			| ( ']' (~(']')) )
			| ( ']' ']' (~('>')) ) )*
			CDATA_END!
                ;

protected
CDATA_START!    :       "<![CDATA["
                ;

protected
CDATA_END!      :       "]]>"
                ;

protected
PROCESSING_INST :	"<?"! 
			( ~('?') | ( '?' (~('>')) ))+
			"?>"! 
		;

XML_DECL_OR_PROCESSING_INST
		:	( "<?" XML ) =>
			XML_DECL { $setType(XML_DECL); }
		|	PROCESSING_INST { $setType(PROCESSING_INST); }
		;

