//*****************************************************************************
//   Copyright (c) 1998 Bernard Sufrin, Oxford.
//   http://www.comlab.ox.ac.uk/oucl/people/bernard.sufrin.html
//*****************************************************************************

//  "Inspired in part by IDVI 1.1 -- the work of Garth A. Dickie"

package figue.jape;

import  java.io.InputStream;
import  java.io.IOException;
import  java.io.EOFException;

/**
        A stream equipped with functions which support the decoding
        of byte-encoded  files. 
        <P>
        All <TT>signed</TT><I>i</I> and  <TT>unsigned</TT><I>i</I> functions
        expect big-endian ordering of the bytes on the input stream. 
*/

public final class ByteCodedInputStream {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/jape/ByteCodedInputStream.java,v $ $revision$ $Date: 1998/11/02 15:55:22 $ Copyright 1998, Bernard Sufrin, Oxford.";

    InputStream is;

    public ByteCodedInputStream( InputStream is ) throws IOException 
    {
        this.is = is;
    }

    public final void close( ) throws IOException 
    {
        is.close( );
    }

    public final int read( ) throws IOException 
    {
        return unsigned1( );
    }

    public final void read( byte[ ] dest ) throws IOException 
    {
        read( dest, 0, dest.length );
    }

    public final void read( byte[ ] dest, int offset, int length ) 
    throws IOException 
    {   int end = offset + length;
        do 
        {   int count = is.read( dest, offset, end - offset );
            if ( count == -1 ) throw new EOFException( );
            offset += count;
        }   while( offset != end );
    }

    public final String string( int length ) throws IOException 
    {
        byte[ ] buffer = new byte[ length ];
        read( buffer );
        return new String( buffer,0,length ); /// +++
        /// return new String( buffer, 0, 0, length ); /// ---
    }

    public final void skip( long count ) throws IOException 
    {
        long skipped;
        do 
        {   skipped = is.skip( count );
            count  -= skipped;
        }   while ( count != 0 && skipped != 0 );

        if ( count != 0 ) throw new EOFException( );
    }

    public final int signed( int count ) throws IOException 
    {
        int result = ( int ) ( byte ) unsigned1( );
        while ( -- count != 0 ) result = ( result << 8 ) | unsigned1( );
        return result;
    }

    public final int signed1( ) throws IOException 
    {
        return ( int ) ( byte ) unsigned1( );
    }

    public final int signed2( ) throws IOException 
    {
        return signed( 2 );
    }

    public final int signed3( ) throws IOException 
    {
        return signed( 3 );
    }

    public final int signed4( ) throws IOException 
    {
        return signed( 4 );
    }

    public final int unsigned( int count ) throws IOException 
    {
        int result = unsigned1( );
        while ( -- count != 0 )  result = ( result << 8 ) | unsigned1( );       
        return result;
    }

    public final int unsigned1( ) throws IOException 
    {
        int result = is.read( );
        if ( result == -1 ) throw new EOFException( );
        return result;
    }

    public final int unsigned2( ) throws IOException 
    {   int result = unsigned1();
        result = (result << 8) | unsigned1();
        return result;
    }

    public final int unsigned3( ) throws IOException 
    {   int result = unsigned1();
        result = (result << 8) | unsigned1();
        result = (result << 8) | unsigned1();
        return result;
    }

    public final int unsigned4( ) throws IOException 
    {   int result = unsigned1();
        result = (result << 8) | unsigned1();
        result = (result << 8) | unsigned1();
        result = (result << 8) | unsigned1();
        return result;
    }
}




