//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.resource;

import figue.internal.Globals;
import figue.error.ErrorHandlerInterface;
import figue.error.ShouldNotBeCalledError;

import java.util.Hashtable;
import java.net.URL;
import java.awt.Graphics;

/**
 * [PENDING: update the comments] ASCIIResourceFactory.
 * @level advanced
 */

public final class ASCIIResourceFactory extends ResourceFactory {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/ASCIIResourceFactory.java,v $ $revision$ $Date: 1998/11/02 15:55:33 $ Copyright 1998 INRIA.";
    
    public ASCIIResourceFactory(ErrorHandlerInterface anErrorHandler,
				URL aPortableFontUrl) {
	super(anErrorHandler);
	_colorRegistry = new Hashtable(Globals.AVERAGE_COLOR_NUMBER,
				       Globals.VERY_FAST_ACCESS);
	_fontRegistry = new Hashtable(Globals.AVERAGE_FONT_NUMBER,
				      Globals.FAST_ACCESS);
	_styleRegistry = new Hashtable(Globals.AVERAGE_STYLE_NUMBER,
				       Globals.VERY_FAST_ACCESS);
	_defaultStyle = null;
	_asciiColorRegistry = new Hashtable(Globals.AVERAGE_COLOR_NUMBER,
					    Globals.VERY_FAST_ACCESS);
	_asciiFontRegistry = new Hashtable(Globals.AVERAGE_FONT_NUMBER,
					   Globals.FAST_ACCESS);
	_portableFontRegistry =  new Hashtable(Globals.AVERAGE_FONT_NUMBER,
					       Globals.FAST_ACCESS);
    }
    
    public final ColorInterface createColor(String aMnemonic,
					    BasicColor aColor)
	throws WrongResourceException {
	return new ASCIIColor(aMnemonic);
    }
    
    public final ColorInterface createColor(String aMnemonic,
					    int red, 
					    int green, 
					    int blue)
	throws WrongResourceException {
	final String theKey = computeKey(red,green,blue);
	ColorInterface theColor = (ColorInterface)(_asciiColorRegistry.get(theKey));
	if ( theColor != null ) {
	    return theColor;
	}
	theColor = new ASCIIColor(aMnemonic);
	_asciiColorRegistry.put(theKey,theColor);
	return theColor;
    }
    
    public final FontInterface createFont(String aMnemonic,
					  FontFamily aFamily,
					  FontStyle aStyle,
					  int aPointSize,
					  boolean anUnderlinedFlag)
	throws WrongResourceException {
	final String theKey = computeKey(aFamily,aStyle,aPointSize,anUnderlinedFlag);
	FontInterface theFont = (FontInterface)(_asciiFontRegistry.get(theKey));
	if ( theFont != null ) {
	    return theFont;
	}
	theFont = new ASCIIFont(aMnemonic,theKey,aFamily,aStyle,aPointSize,anUnderlinedFlag);
	_asciiFontRegistry.put(theKey,theFont);
	return theFont;
    }
    
    protected final Hashtable getColorRegistry() {
	return _colorRegistry;
    }
    protected final Hashtable getFontRegistry() {
	return _fontRegistry;
    }
    protected final Hashtable getStyleRegistry() {
	return _styleRegistry;
    }
    
    protected final StyleInterface getDefaultStyle() {
	return _defaultStyle;
    }
    
    protected final void setDefaultStyle(StyleInterface aStyle) {
	_defaultStyle = aStyle;
    }

    protected final Hashtable getPortableFontRegistry() {
	return _portableFontRegistry;
    }
    
    public final PortableFontInterface createPortableFont(String aMnemonic,
							  String aName,
							  int aDpi,
							  double aScale)
	throws WrongResourceException {
	return new ASCIIPortableFont(aMnemonic,aName,aDpi,aScale);
    }
    
    public final void dispose() {
	_colorRegistry.clear();
	_colorRegistry = null;
	_fontRegistry.clear();
	_fontRegistry = null;
	_styleRegistry.clear();
	_styleRegistry = null;
	_asciiColorRegistry.clear();
	_asciiColorRegistry = null;
	_asciiFontRegistry.clear();
	_asciiFontRegistry = null;
	_portableFontRegistry.clear();
    }
    
    private Hashtable _colorRegistry;
    private Hashtable _fontRegistry;
    private Hashtable _styleRegistry;
    private StyleInterface _defaultStyle;
    private Hashtable _asciiColorRegistry;
    private Hashtable _asciiFontRegistry;
    private Hashtable _portableFontRegistry;
    
}

final class ASCIIColor extends Color {  

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/ASCIIResourceFactory.java,v $ $revision$ $Date: 1998/11/02 15:55:33 $ Copyright 1998 INRIA.";
    
    public ASCIIColor(String aMnemonic) { 
	super(aMnemonic);
    }
    
    public final int getRed() {
	return 0;
    }
    public final int getGreen() {
	return 0;
    }
    public final int getBlue() {
	return 0;
    }

}

final class ASCIIFont extends Font {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/ASCIIResourceFactory.java,v $ $revision$ $Date: 1998/11/02 15:55:33 $ Copyright 1998 INRIA.";
    
    public ASCIIFont(String aMnemonic,
		     String aKey,
		     FontFamily aFamily,
		     FontStyle aStyle,
		     int aPointSize,
		     boolean anUnderlinedFlag) {
	super(aMnemonic,aKey,aFamily,aStyle,aPointSize,anUnderlinedFlag);
    }
    
    public final boolean equals(Object anObject) {
	if ( this == anObject ) {
	    return true;
	}
	if ( getClass() != anObject.getClass() ) {
	    return false;
	}
	return getKey().equals(((ASCIIFont)anObject).getKey());
    }
    
}

final class ASCIIPortableFont implements PortableFontInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/ASCIIResourceFactory.java,v $ $revision$ $Date: 1998/11/02 15:55:33 $ Copyright 1998 INRIA.";

    public ASCIIPortableFont(String aMnemonic, String aName, int aDpi, double aScale) {
	_name = aName;
	_dpi = aDpi;
	_scale = aScale;
    }

    public final String getName() {
	return _name;
    }
    
    public final int getDpi() {
	return _dpi;
    }
    
    public final double getScale() {
	return _scale;
    }

    public final void draw(Graphics aContext, String aString, int x, int y) {
	throw new ShouldNotBeCalledError();
    }

    public final FontMetricInterface getFontMetric() {
	if ( _metric == null ) {
	    _metric = new ASCIIPortableFontMetric();
	}
	return _metric;
    }

    private String _name;
    private int _dpi;
    private double _scale;
    private FontMetricInterface _metric;

}

final class ASCIIPortableFontMetric implements FontMetricInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/ASCIIResourceFactory.java,v $ $revision$ $Date: 1998/11/02 15:55:33 $ Copyright 1998 INRIA.";
    
    public ASCIIPortableFontMetric() {
    }
    
    public final int getAscent() {
	return 1;
    }
    
    public final int getDescent() {
	return 0;
    }
    
    public final int bytesWidth(byte[] aByteArray) {
	return aByteArray.length;
    }
    
    public final int stringWidth(String aString) {
	return aString.length();
    }
    
    public final int getHeight() {
	return 1;
    }
    
    public final FontInterface getFont() {
	throw new ShouldNotBeCalledError(); /// BUG NOT YET IMPLEMENTED
    }
    
    public final boolean equals(Object anObject) {
	if ( this == anObject ) {
	    return true;
	}
	if ( getClass() != anObject.getClass() ) {
	    return false;
	}
	return getFont().equals(((ASCIIPortableFontMetric)anObject).getFont());
    }
    
}
