//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.selection;

import figue.FacadeInterface;
import figue.Priority;
import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.path.TouchSelection;
import figue.path.Path;
import figue.resource.ColorDefered;
import figue.resource.ColorInterface;
import figue.resource.ColorInitializer;
import figue.resource.BackgroundInitializer;
import figue.error.ErrorHandler;
import figue.error.ErrorHandlerInterface;
import figue.error.Error;
import figue.error.Warning;
import figue.error.ShouldNotBeCalledError;
import figue.io.Utility;

import java.io.Writer;
import java.io.IOException;

/**
 * [PENDING: update the comments] Selection.
 * @level advanced
 */

public final class Selection extends ErrorHandler implements SelectionInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/selection/Selection.java,v $ $revision$ $Date: 1998/11/02 15:55:45 $ Copyright 1998 INRIA.";

    public Selection(String aName,
		     Priority aPriority,
		     ColorInterface aColor,
		     ColorInterface aBackground,
		     FacadeInterface aFacade) {
	_name = aName;
	_hidden = false;
	_priority = aPriority;
	_priorityEnum = aPriority.getEnum();
	_background = aBackground;
	_color = aColor;
	_highlighter = DefaultHighlighter.INSTANCE;
	_facade = aFacade;
    }

    public Selection(String aName,
		     Priority aPriority,
		     String aColorName,
		     String aBackgroundName,
		     FacadeInterface aFacade) {
	_name = aName;
	_hidden = false;
	_priority = aPriority;
	_priorityEnum = aPriority.getEnum();
	if ( aColorName != null ) {
	    _color = new ColorDefered(aColorName,
				      new ColorInitializer(this));
	}
	else {
	    _color = null;
	}
	if ( aBackgroundName != null ) {
	    _background = new ColorDefered(aBackgroundName,
					   new BackgroundInitializer(this));
	}
	else {
	    _background = null;
	}
	_highlighter = DefaultHighlighter.INSTANCE;
	_facade = aFacade;
    }

    public final String getName() {
	return _name;
    }

    public final void setVisible(boolean aFlag) {
	_hidden = ! aFlag;
	final PathInterface thePath = computeTouchSelectionPath(_facade.getSelectionPath());
	if ( thePath != null ) {
	    _facade.updateSelectionAndRedisplay(thePath);
	}
    }

    private final PathInterface computeTouchSelectionPath(SelectionPathInterface aSelectionPath) {
	PathInterface thePath = new Path(aSelectionPath.getRank());
	boolean theFindFlag = false;
	if ( aSelectionPath.contains(this) ) {
	    theFindFlag = true;
	    thePath.addOperation(TouchSelection.INSTANCE);
	}
	final int theSize = aSelectionPath.numberOfChildren();
	PathInterface theCurrentResult = null;
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentResult = computeTouchSelectionPath(aSelectionPath.getChild(theRank));
	    if ( theCurrentResult != null ) {
		theFindFlag = true;
		thePath.addChild(theCurrentResult);
	    }
	}
	if ( theFindFlag ) {
	    return thePath;
	}
	else {
	    return null;
	}    
    }
    
    public final boolean isVisible() {
	return ! _hidden;
    }

    public final int getPriority() {
	return _priorityEnum;
    }
    
    public final ColorInterface getColor() {
	return _color;
    }
    
    public final void setColor(ColorInterface aColor) {
	_color = aColor;
    }
    
    public final ColorInterface getBackground() {
	return _background;
    }
    
    public final void setBackground(ColorInterface aColor) {
	_background= aColor;
    }

    public final HighlighterInterface getHighlighter() {
	return _highlighter;
    }

    public final void setHighlighter(HighlighterInterface anHighlighter) {
	_highlighter = anHighlighter;
    }
    
    public final FacadeInterface getFacade() {
	return _facade;
    }
    
    public final boolean hasNextErrorHandler() {
	return ( getFacade() != null );
    }
    
    public final ErrorHandlerInterface getNextErrorHandler() {
	return getFacade();
    }
    
    public final void manageFatalError(Error anError) {
	throw new ShouldNotBeCalledError();
    }
    
    public final void manageWarning(Warning aWarning) {
	throw new ShouldNotBeCalledError();
    }
    
    public final boolean canHandleError() {
	return false;
    }
    
    public final String toString() {
	StringBuffer theResult = new StringBuffer("Selection<");
	theResult.append(_name).append('/').append(_priority).append(">");
	return theResult.toString();
    }
    
    public void writeState(Writer anOutput) throws IOException {
	anOutput.write("<Selection Name=\"");
	Utility.writeStringAsXmlValue(anOutput,_name);
	anOutput.write("\" Visible=\"");
	if ( _hidden ) {
	    anOutput.write("false");
	}
	else {
	    anOutput.write("true");
	}
	anOutput.write("\" Priority=\"");
	Utility.writeStringAsXmlValue(anOutput,_priority.getName());
	anOutput.write("\"");
	if ( _background != null ) {
	    anOutput.write(" Background=\"");
	    Utility.writeStringAsXmlValue(anOutput,_background.getName());
	    anOutput.write("\"");
	}
	if ( _color != null ) {
	    anOutput.write(" Foreground=\"");
	    Utility.writeStringAsXmlValue(anOutput,_color.getName());
	    anOutput.write("\"");
	}
	anOutput.write("/>");
    }
    
    private String _name;
    private boolean _hidden;
    private Priority _priority;
    private int _priorityEnum;
    private ColorInterface _background;
    private ColorInterface _color;
    private HighlighterInterface _highlighter;
    private FacadeInterface _facade;
    
}
